@@ -18,11 +18,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -44,8 +43,6 @@
 #endif
 
 
-extern int errno;
-
 #define MAX_LINE_LENGTH 4096
 #define FIELD_DELIMITER ':'
 
@@ -93,7 +90,7 @@ void makebdb_new(const makebdb_options_t options){
   }
   if ((ret=dbp->open(
     dbp,
-#ifdef HAVE_BDB_4_1
+#ifdef HAVE_BDB_OPEN_7
     NULL,
 #endif
     options.mapname,
@@ -178,7 +175,7 @@ void makebdb_undo(makebdb_options_t options){
   }
   if ((ret=dbp->open(
     dbp,
-#ifdef HAVE_BDB_4_1
+#ifdef HAVE_BDB_OPEN_7
     NULL,
 #endif
     options.mapname,
