@@ -37,7 +37,7 @@ set_signal(int signum, void (*handler)(int))
 
 
 static void 
-usage(int exit_status)
+daemon_usage(int exit_status)
 {
 	fprintf(exit_status?stderr:stdout,
 			"perdition test-server version %s Copyright Horms\n"
@@ -69,10 +69,15 @@ main(int argc, char **argv)
 	vanessa_logger_t *vl;
 	perdition_packet_t *req_packet;
 	perdition_packet_t *rsp_packet;
-	perdition_packet_str_t user;
+	perdition_packet_str_t saddr;
+	perdition_packet_str_t sport;
+	perdition_packet_str_t daddr;
+	perdition_packet_str_t dport;
+	perdition_packet_str_t key;
+	perdition_packet_str_t domain_delimiter;
 	perdition_packet_str_t server;
-	char *server_socket;
-	char *response;
+	char *server_socket = NULL;
+	char *response = NULL;
 
 	vl = vanessa_logger_openlog_filehandle(stderr, "test-server",
 			LOG_DEBUG, 0);
@@ -87,11 +92,11 @@ main(int argc, char **argv)
 		response = argv[2];
 	}
 	else {
-		usage(-1);
+		daemon_usage(-1);
 	}
 
 	if(!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help")) {
-		usage(0);
+		daemon_usage(0);
 	}
 
 	set_signal(SIGHUP,  SIG_IGN);
@@ -131,7 +136,7 @@ main(int argc, char **argv)
 
 	memset(&unaddr, 0, sizeof(struct sockaddr_un));
 	unaddr.sun_family = AF_UNIX;
-	strncpy(unaddr.sun_path, un.name, sizeof(struct sockaddr_un));
+	strncpy(unaddr.sun_path, un.name, PERDITION_UN_STR_LEN);
 
 	unlink(unaddr.sun_path);
 	if(bind(un.fd, (struct sockaddr *) &unaddr, 
@@ -160,15 +165,20 @@ main(int argc, char **argv)
 		VANESSA_LOGGER_DEBUG_UNSAFE("%d bytes recieved from %s", 
 				bytes_recv, unaddr.sun_path);
 
-		if(perdition_packet_verify_v1_req(req_packet, bytes_recv, 
-					&user, NULL) < 0) {
-			VANESSA_LOGGER_DEBUG("perdition_packet_verify_v1_req");
+		if (perdition_packet_verify_v1_req(req_packet, bytes_recv,
+					           &key, &domain_delimiter) &&
+		    perdition_packet_verify_v1_str_req(req_packet, bytes_recv,
+						       &saddr, &sport,
+						       &daddr, &dport,
+					               &key,
+						       &domain_delimiter) < 0) {
+			VANESSA_LOGGER_DEBUG("perdition_packet_verify_v1_*");
 			continue;
 		}
 
 		PERDITION_PACKET_STR_PACK(server, response);
 		if(perdition_packet_init_v1_rsp(&rsp_packet, 0,
-				&user, &server, NULL) < 0) {
+				&key, &server, NULL) < 0) {
 			VANESSA_LOGGER_DEBUG("perdition_packet_init_v1_req");
 			continue;
 		}
