@@ -13,6 +13,14 @@
  * Mail retrieval proxy server, LDAP support
  * Copyright (C) 1999-2005  ChrisS and Horms
  * 
+ * Contributions:
+ *
+ *
+ * 	Oct/2007: Confederacao SICREDI - www.sicredi.com.br
+ *                Felipe Damasio - felipe_damasio@sicredi.com.br
+ *                Tiago A. Wegner - tiago_wegner@sicredi.com.br
+ * 	- Fix the LDAP connection using the proper string for ldap_initialize
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -22,11 +30,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -34,18 +41,16 @@
 #include "config.h"
 #endif
 
+#include "perdition_globals.h"
 #include "perditiondb_ldap.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-
 static char *pldap_filter = NULL;
 
-#ifdef WITH_LDAP_SET_OPTION
 static int pldap_version = PERDITIONDB_LDAP_VERSION;
-#endif /* WITH_LDAP_SET_OPTION */
 
 /**********************************************************************
  * dbserver_init
@@ -63,19 +68,51 @@ static int pldap_version = PERDITIONDB_LDAP_VERSION;
 
 int dbserver_init(char *options_str)
 {
-	if (options_str == NULL) {
-		options_str = PERDITIONDB_LDAP_DEFAULT_URL;
+	char *str = options_str;
+
+	if (str && *str) {
+		if (*(str+1) == '\0') {
+			pldap_version = (int) *str - '0';
+			str++;
+		}
+		else if (*(str+1) == ':') {
+			pldap_version = (int) *str - '0';
+			str += 2;
+		}
+	}
+
+	if (pldap_version < LDAP_VERSION_MIN || 
+			pldap_version > LDAP_VERSION_MAX) {
+		VANESSA_LOGGER_DEBUG_RAW_UNSAFE("Requested ldap version (%c): "
+				"is out of range. Must be a numeric value "
+				"between %d and %d", *options_str, 
+				LDAP_VERSION_MIN, LDAP_VERSION_MAX);
+		return -1;
+	}
+
+#ifdef WITH_LDAP_SET_OPTION
+	VANESSA_LOGGER_DEBUG_RAW_UNSAFE("Using LDAP version: %d", 
+			pldap_version);
+#else
+	VANESSA_LOGGER_DEBUG_RAW_UNSAFE(
+			"The ldap library perdition was compiled against "
+			"does not support setting of the ldap version. "
+			"Using the default version: %d", LDAP_VERSION);
+#endif
+
+	if (!str || !*str) {
+		str = PERDITIONDB_LDAP_DEFAULT_URL;
 	}
 
 	/*
 	 * Some checks to see if the URL is sane in LDAP terms
 	 */
-	if (ldap_is_ldap_url(options_str) == 0) {
+	if (ldap_is_ldap_url(str) == 0) {
 		VANESSA_LOGGER_DEBUG("ldap_is_ldap_url: not an LDAP URL");
 		return (-1);
 	}
 
-	pldap_filter = strdup(options_str);
+	pldap_filter = strdup(str);
 	if(!pldap_filter) {
 		VANESSA_LOGGER_DEBUG_ERRNO("pldap_filter");
 	}
@@ -285,25 +322,26 @@ static int pldap_scan_exts(char **exts, char **bindname, char **xbindpw)
 
 
 /**********************************************************************
- * dbserver_get
+ * dbserver_get2
  * Read the server (value) from an LDAP directory given the user (key)
  * pre: key_str: Key as a null terminated string
- *      options_str: Options string. 
+ *      options_str: Options string.
  *                   Ignored if NULL
  *                   Used as the map to open otherwise
- *      str_return: value is returned here
- *      len_return: length of value is returned here
- * post: The str_key is looked up in the gdbm map and the
- *       corresponding value is returned in str_return and len_return.
+ *      user_str: string for user is returned here
+ *      server_str: string for server is returned here
+ *      port_str: string for port is returned here
+ * post: The str_key is looked up in the ldap database and the
+ *       corresponding values are returned in user_str, server_str and
+ *       port_str.
  * return:  0 on success
  *         -1 on LDAP access error
  *         -2 if key cannot be found in map
  *         -3 on other error
  **********************************************************************/
 
-int dbserver_get(const char *key_str,
-		 const char *options_str,
-		 char **str_return, int *len_return)
+int dbserver_get2(const char *key_str, const char *options_str,
+		 char **user_str, char **server_str, char **port_str)
 {
 	LDAPURLDesc *lud = NULL;
 	LDAP *connection = NULL;
@@ -319,11 +357,7 @@ int dbserver_get(const char *key_str,
 	char **returns = NULL;
 	char *binddn = NULL;
 	char *bindpw = NULL;
-
-	extern options_t opt;
-
-	*len_return = 0;
-
+	char *ldap_connect = NULL;
 
 	/* get filter string */
 	if (pldap_get_filter(key_str, pldap_filter, &lud) < 0) {
@@ -334,9 +368,22 @@ int dbserver_get(const char *key_str,
 	}
 
 	/* Open LDAP connection */
-#if 0
-//#if defined(LDAP_API_FEATURE_X_OPENLDAP) && (LDAP_API_VERSION > 2000)
-	err = ldap_initialize(&connection, pldap_filter);
+#if defined(LDAP_API_FEATURE_X_OPENLDAP) && (LDAP_API_VERSION > 2000)
+	/*
+         * This '+6' on calloc is the worst case scenario of a non-default
+         * LDAP port: 65535 and such. The extra byte is for '\0'
+         */
+	ldap_connect = calloc(strlen (lud->lud_scheme) +
+			      strlen (lud->lud_host) + 6, sizeof (char));
+	if (!ldap_connect)
+		goto leave;
+	if (lud->lud_port != LDAP_PORT)
+		sprintf(ldap_connect, "%s://%s:%d", lud->lud_scheme,
+			lud->lud_host, lud->lud_port);
+	else
+		sprintf(ldap_connect, "%s://%s", lud->lud_scheme,
+			lud->lud_host);
+	err = ldap_initialize(&connection, ldap_connect);
 	if (err != LDAP_SUCCESS) {
 		VANESSA_LOGGER_DEBUG_UNSAFE("ldap_initialize: %s",
 				ldap_err2string(err));
@@ -393,7 +440,11 @@ int dbserver_get(const char *key_str,
 	/* Perform the search */
 	err = ldap_search_s(connection, lud->lud_dn, lud->lud_scope,
 			   lud->lud_filter, lud->lud_attrs, 0, &res);
-	if (err != LDAP_SUCCESS) {
+        /* Simon Fraser has observed that when using openldap 2.4.11 on
+	 * Debian Etch (2.1.30) that ldap_search_s() may return
+	 * LDAP_SERVER_DOWN even if the search is successful.
+	 */
+	if (err != LDAP_SUCCESS && err != LDAP_SERVER_DOWN) {
 		VANESSA_LOGGER_DEBUG_UNSAFE("ldap_search_s: %s",
 				ldap_err2string(err));
 		goto leave;
@@ -432,7 +483,6 @@ int dbserver_get(const char *key_str,
 		goto leave;
 	}
 
-	*len_return = 0;
 	for (pstr = ldap_first_attribute(connection, mptr, &ber);
 	     pstr != NULL;
 	     pstr = ldap_next_attribute(connection, mptr, ber)) {
@@ -442,7 +492,6 @@ int dbserver_get(const char *key_str,
 			if (strcasecmp(lud->lud_attrs[count], pstr) != 0) {
 				continue;
 			}
-			*len_return += strlen(*bv_val);
 			if (returns[count] != NULL) {
 				free(returns[count]);
 			}
@@ -465,41 +514,20 @@ int dbserver_get(const char *key_str,
 	ber_free(ber, 0);
 	ber = NULL;
 
-	/* Add in some extra for the separators and terminating NULL */
-	*len_return += 2 + strlen(opt.domain_delimiter);
-
-	if ((*str_return = (char *) calloc(1, *len_return)) == NULL) {
-		VANESSA_LOGGER_DEBUG_ERRNO("str_return calloc");
-		status = -3;
-		goto leave;
-	}
-
 	/* Build the return string */
-	if (attrcount > 0 && returns[0]) {
-		strcat(*str_return, returns[0]);
-	}
-	if (attrcount > 1 && returns[1]) {
-		if (returns[0]) {
-			strcat(*str_return, opt.domain_delimiter);
-		}
-		strcat(*str_return, returns[1]);
-	}
-	if (attrcount > 2 && returns[2] && returns[1]) {
-		strcat(*str_return, ":");
-		strcat(*str_return, returns[2]);
-	}
-	for (count = 0; count < attrcount; count++) {
-		if (!returns[count]) {
-			continue;
-		}
-		free(returns[count]);
-		returns[count] = NULL;
-	}
+	if (returns[0])
+		*user_str = returns[0];
+	if (returns[1])
+		*server_str = returns[1];
+	if (returns[2])
+		*port_str = returns[2];
 
 	status = 0;
 
       leave:
-	if (returns) {
+	if (ldap_connect)
+		free(ldap_connect);
+	if (returns && status) {
 		for (count = 0; count < attrcount; count++)
 			if (returns[count] != NULL)
 				free(returns[count]);
