@@ -17,11 +17,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -67,7 +66,7 @@ token_t *imap4_tag_create(void)
 		return(NULL);
 	}
 
-	token_assign(tag, buf, strlen(buf), TOKEN_NONE);
+	token_assign(tag, (unsigned char *)buf, strlen(buf), TOKEN_NONE);
 
 	return(tag);
 }
@@ -84,7 +83,7 @@ token_t *imap4_tag_create(void)
 
 void imap4_tag_inc(token_t *tag) 
 {
-	char *buf;
+	unsigned char *buf;
 	size_t len;
 	int c;
 	int last;
