@@ -17,11 +17,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -53,8 +52,6 @@ static flag_t imap4s_encryption(flag_t ssl_flags);
 char *imap4s_type[]={IMAP4_OK, IMAP4_NO, IMAP4_BAD};
 
 protocol_t *imap4s_initialise_protocol(protocol_t *protocol){
-  extern char *imap4s_type[];
-
   protocol->type = imap4s_type;
   protocol->write = imap4_write;
   protocol->greeting_string = IMAP4_GREETING;
