@@ -18,11 +18,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -34,6 +33,7 @@
 
 #include "options.h"
 #include "config_file.h"
+#include "perdition_globals.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
@@ -45,6 +45,73 @@
 options_t opt;
 
 
+/**********************************************************************
+ * log_passwd_to_str
+ * Get the string value of a log_passwd mode
+ * pre: x: integer mode
+ * return: mode string
+ *         NULL on error
+ **********************************************************************/
+
+static const char *
+log_passwd_to_str(int x)
+{
+	switch(x){
+      		case LOG_PASSWD_NEVER:
+	      		return LOG_PASSWD_NEVER_STR;
+      		case LOG_PASSWD_OK:
+	      		return LOG_PASSWD_OK_STR;
+      		case LOG_PASSWD_FAIL:
+			return LOG_PASSWD_FAIL_STR;
+      		case LOG_PASSWD_ALWAYS:
+			return LOG_PASSWD_ALWAYS_STR;
+	}
+	return NULL;
+}
+
+
+/**********************************************************************
+ * log_passwd_from_str
+ * Get the integer value of a log_passwd mode
+ * pre: str: string mode
+ * return: mode integer
+ *         -1 on error
+ **********************************************************************/
+
+static int
+log_passwd_from_str(const char *str)
+{
+	if (!strcasecmp(str, LOG_PASSWD_NEVER_STR))
+		return LOG_PASSWD_NEVER;
+	else if (!strcasecmp(str, LOG_PASSWD_FAIL_STR))
+		return LOG_PASSWD_FAIL;
+	else if (!strcasecmp(str, LOG_PASSWD_OK_STR))
+		return LOG_PASSWD_OK;
+	else if (!strcasecmp(str, LOG_PASSWD_ALWAYS_STR))
+		return LOG_PASSWD_ALWAYS;
+	return -1;
+}
+
+/**********************************************************************
+ * options_set_mask
+ * Set the options mask
+ * pre: mask: pointer to current mask that may be modified
+ *      mask_entry: value to or with opt->mask
+ *      flag: flags
+ * post: mask is added if flags permit
+ * return: 1 if mask is added
+ *         0 otherwise
+ **********************************************************************/
+
+static int 
+options_set_mask(flag_t *mask, flag_t mask_entry, flag_t flag)
+{
+  if(flag&OPT_USE_MASK && (*mask)&mask_entry) return(0);
+  if(flag&OPT_SET_MASK) (*mask)|=mask_entry;
+  return(1);
+}
+
+
 /***********************************************************************
  * opt_p
  * Assign an option that is a char *
@@ -59,11 +126,16 @@ options_t opt;
  *       Else no change.
  ***********************************************************************/
 
-#define opt_p(opt, value, mask, mask_entry, flag) \
-  if(options_set_mask(&(mask), mask_entry, flag)){ \
-    if(!((flag)&OPT_NOT_SET) && opt!=NULL){ free(opt); } \
-    opt=(value==NULL)?NULL:strdup(value); \
-  }
+static void
+opt_p(char **opt, const char *value, 
+		flag_t *mask, flag_t mask_entry, flag_t flag)
+{
+      	if (!options_set_mask(mask, mask_entry, flag))
+      		return;
+  	if (!((flag)&OPT_NOT_SET) && *opt != NULL)
+		free(*opt); 
+	*opt = (!value) ? NULL : strdup(value);
+}
 
 
 /***********************************************************************
@@ -80,12 +152,36 @@ options_t opt;
  *       Else no change.
  ***********************************************************************/
 
-#define opt_i(opt, value, mask, mask_entry, flag) \
-  if(options_set_mask(&(mask), mask_entry, flag)){ \
-    opt=value; \
-  }
+static void
+opt_i(int *opt, int value, flag_t *mask, flag_t mask_entry, flag_t flag)
+{
+      	if (options_set_mask(mask, mask_entry, flag))
+		*opt = value;
+}
 
 
+/***********************************************************************
+ * opt_i_or
+ * Assign an option that is an int
+ * The value assigned will be the logical or of the current value 
+ * of the option, and the value paramater
+ * pre: opt: option to assign
+ *      value: value to assign to opt
+ *      mask:  current option mask
+ *      mask_entry: entry of this option in the option mask
+ *      flag:  flags as per options.h
+ * post: If the mask and options allow as per options_set_mask()
+ *       value is assigned to opt. 
+ *       The mask may also be altered as per options_set_mask()
+ *       Else no change.
+ ***********************************************************************/
+
+static void
+opt_i_or(int *opt, int value, flag_t *mask, flag_t mask_entry, flag_t flag)
+{
+	opt_i(opt, *opt|value, mask, mask_entry, flag);
+}
+
 /***********************************************************************
  * opt_da
  * Assign an option that is a vanessa_dynamic_array *
@@ -100,36 +196,35 @@ options_t opt;
  *       Else no change.
  ***********************************************************************/
 
-#define opt_da(opt, value, mask, mask_entry, flag) \
-  if(options_set_mask(&(mask), mask_entry, flag)){ \
-    if(!((flag)&OPT_NOT_SET) && opt!=NULL){ \
-      vanessa_dynamic_array_destroy(opt); \
-    } \
-    opt=value; \
-  } \
-
-
-#define opt_i_or(opt, value, mask, mask_entry, flag) \
-  opt_i(opt, opt|value, mask, mask_entry, flag)
+static void
+opt_da(vanessa_dynamic_array_t **opt, vanessa_dynamic_array_t *value, 
+		flag_t *mask, flag_t mask_entry, flag_t flag)
+{
+	if (!options_set_mask(mask, mask_entry, flag))
+		return;
+	if (!((flag)&OPT_NOT_SET) && *opt != NULL)
+		vanessa_dynamic_array_destroy(*opt);
+	*opt = value;
+}
 
 #define OPT_MODIFY_USERNAME(opt, mask, mask_entry, flag, id_str) \
   if(strcasecmp(optarg_copy, "all")==0){ \
-    opt_i_or(opt, STATE_ALL, mask, mask_entry, flag); \
+    opt_i_or(&opt, STATE_ALL, &mask, mask_entry, flag); \
   } \
   else if( \
     strcasecmp(optarg_copy, "servername_lookup")==0 || \
     strcasecmp(optarg_copy, "server_lookup")==0 \
   ){ \
-    opt_i_or(opt, STATE_GET_SERVER, mask, mask_entry, flag); \
+    opt_i_or(&opt, STATE_GET_SERVER, &mask, mask_entry, flag); \
   } \
   else if( \
     strcasecmp(optarg_copy, "local_authentication")==0 || \
     strcasecmp(optarg_copy, "local_auth")==0 \
   ){ \
-    opt_i_or(opt, STATE_LOCAL_AUTH, mask, mask_entry, flag); \
+    opt_i_or(&opt, STATE_LOCAL_AUTH, &mask, mask_entry, flag); \
   } \
   else if(strcasecmp(optarg_copy, "remote_login")==0){ \
-    opt_i_or(opt, STATE_REMOTE_LOGIN, mask, mask_entry, flag); \
+    opt_i_or(&opt, STATE_REMOTE_LOGIN, &mask, mask_entry, flag); \
   } \
   else { \
     VANESSA_LOGGER_ERR_UNSAFE("unknown state for %s: %s", id_str, optarg_copy); \
@@ -191,13 +286,13 @@ options_t opt;
        new=SSL_MODE_TLS_OUTGOING; \
     } \
     else if(strcasecmp(optarg_copy, "tls_listen_force")==0){ \
-       new=SSL_MODE_TLS_LISTEN_FORCE; \
+       new=(SSL_MODE_TLS_LISTEN_FORCE|SSL_MODE_TLS_LISTEN); \
     } \
     else if(strcasecmp(optarg_copy, "tls_outgoing_force")==0){ \
-       new=SSL_MODE_TLS_OUTGOING_FORCE; \
+       new=(SSL_MODE_TLS_OUTGOING_FORCE|SSL_MODE_TLS_OUTGOING); \
     } \
     else if(strcasecmp(optarg_copy, "tls_all_force")==0){ \
-       new=SSL_MODE_TLS_ALL_FORCE; \
+       new=(SSL_MODE_TLS_ALL_FORCE|SSL_MODE_TLS_ALL); \
     } \
     else if(strcasecmp(optarg_copy, "tls_all")==0){ \
        new=SSL_MODE_TLS_ALL; \
@@ -218,7 +313,7 @@ options_t opt;
       VANESSA_LOGGER_DEBUG_RAW("invalid ssl_mode combination"); \
       if(f&OPT_ERR) vanessa_socket_daemon_exit_cleanly(-1); \
     } \
-    opt_i_or(opt.ssl_mode, new, opt.ssl_mask, MASK_SSL_MODE, f); \
+    opt_i_or(&(opt.ssl_mode), new, &(opt.ssl_mask), MASK_SSL_MODE, f); \
   }
 #else /* WITH_SSL_SUPPORT */
 #define NO_SSL_OPT(_opt)                                                     \
@@ -260,6 +355,8 @@ int options(int argc, char **argv, flag_t f){
   {
     {"add_domain",                  'A',  POPT_ARG_STRING, NULL, 'A'},
     {"authenticate_in",             'a',  POPT_ARG_NONE,   NULL, 'a'},
+    {"authenticate_timeout",        '\0', POPT_ARG_STRING, NULL,
+     TAG_AUTHENTICATE_TIMEOUT},
     {"no_bind_banner",              'B',  POPT_ARG_NONE,   NULL, 'B'},
     {"bind_address",                'b',  POPT_ARG_STRING, NULL, 'b'},
     {"connection_logging",          'C',  POPT_ARG_NONE,   NULL, 'C'},
@@ -277,11 +374,10 @@ int options(int argc, char **argv, flag_t f){
     {"capability",                  'I',  POPT_ARG_STRING, NULL, 'I'},
     {"imap_capability",             'I',  POPT_ARG_STRING, NULL, 'I'},
     {"pop_capability",              'I',  POPT_ARG_STRING, NULL, 'I'},
-    {"jain",                        'j',  POPT_ARG_NONE,   NULL, 'j'},
-    {"jane",                        'j',  POPT_ARG_NONE,   NULL, 'j'},
-    {"jayne",                       'j',  POPT_ARG_NONE,   NULL, 'j'},
     {"connection_limit",            'L',  POPT_ARG_STRING, NULL, 'L'},
     {"listen_port",                 'l',  POPT_ARG_STRING, NULL, 'l'},
+    {"log_passwd",                  '\0', POPT_ARG_STRING, NULL, 
+      TAG_LOG_PASSWD},
     {"map_library",                 'M',  POPT_ARG_STRING, NULL, 'M'},
     {"map_library_opt",             'm',  POPT_ARG_STRING, NULL, 'm'},
     {"no_lookup",                   'n',  POPT_ARG_NONE,   NULL, 'n'},
@@ -345,68 +441,78 @@ int options(int argc, char **argv, flag_t f){
 
   /* i is used as a dummy variable */
   if(f&OPT_SET_DEFAULT){
-    opt_i(opt.add_domain,      DEFAULT_ADD_DOMAIN,          i, 0, OPT_NOT_SET);
+    opt_i(&(opt.add_domain), DEFAULT_ADD_DOMAIN, &i, 0, OPT_NOT_SET);
     opt.add_domain_strip_depth = DEFAULT_ADD_DOMAIN_STRIP_DEPTH;
 #ifdef WITH_PAM_SUPPORT
-    opt_i(opt.authenticate_in, DEFAULT_AUTHENTICATE_IN,     i, 0, OPT_NOT_SET);
+    opt_i(&(opt.authenticate_in), DEFAULT_AUTHENTICATE_IN, &i, 0, OPT_NOT_SET);
 #endif /* WITH_PAM_SUPPORT */
-    opt_i(opt.no_bind_banner,  DEFAULT_NO_BIND_BANNER,      i, 0, OPT_NOT_SET);
-    opt_i(opt.client_server_specification, DEFAULT_CLIENT_SERVER_SPECIFICATION,
-                                                            i, 0, OPT_NOT_SET);
-    opt_i(opt.connection_limit,DEFAULT_CONNECTION_LIMIT,    i, 0, OPT_NOT_SET);
-    opt_i(opt.connection_logging,DEFAULT_CONNECTION_LOGGING,i, 0, OPT_NOT_SET);
-    opt_i(opt.debug,           DEFAULT_DEBUG,               i, 0, OPT_NOT_SET);
-    opt_i(opt.inetd_mode,      DEFAULT_INETD_MODE,          i, 0, OPT_NOT_SET);
+    opt_i(&(opt.authenticate_timeout), DEFAULT_AUTHENTICATE_TIMEOUT,
+	  &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.no_bind_banner), DEFAULT_NO_BIND_BANNER,
+		    &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.client_server_specification), 
+		    DEFAULT_CLIENT_SERVER_SPECIFICATION, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.connection_limit), DEFAULT_CONNECTION_LIMIT, 
+		    &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.connection_logging), DEFAULT_CONNECTION_LOGGING,
+		    &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.debug), DEFAULT_DEBUG, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.inetd_mode), DEFAULT_INETD_MODE, &i, 0, OPT_NOT_SET);
     if(!(f&OPT_FILE) && !strcmp("perdition.imap4", basename)){
-      opt_i(opt.protocol,      PROTOCOL_IMAP4,              i, 0, OPT_NOT_SET);
+      opt_i(&(opt.protocol), PROTOCOL_IMAP4, &i, 0, OPT_NOT_SET);
     }
     else if(!(f&OPT_FILE) && !strcmp("perdition.imap4s", basename)){
-      opt_i(opt.protocol,      PROTOCOL_IMAP4S,             i, 0, OPT_NOT_SET);
+      opt_i(&(opt.protocol), PROTOCOL_IMAP4S, &i, 0, OPT_NOT_SET);
     }
     else if(!(f&OPT_FILE) && !strcmp("perdition.imaps", basename)){
-      opt_i(opt.protocol,      PROTOCOL_IMAP4S,             i, 0, OPT_NOT_SET);
+      opt_i(&(opt.protocol), PROTOCOL_IMAP4S, &i, 0, OPT_NOT_SET);
     }
     else if(!(f&OPT_FILE) && !strcmp("perdition.pop3", basename)){
-      opt_i(opt.protocol,      PROTOCOL_POP3,               i, 0, OPT_NOT_SET);
+      opt_i(&(opt.protocol), PROTOCOL_POP3, &i, 0, OPT_NOT_SET);
     }
     else if(!(f&OPT_FILE) && !strcmp("perdition.pop3s", basename)){
-      opt_i(opt.protocol,      PROTOCOL_POP3S,              i, 0, OPT_NOT_SET);
+      opt_i(&(opt.protocol), PROTOCOL_POP3S, &i, 0, OPT_NOT_SET);
     }
     else {
-      opt_i(opt.protocol,      DEFAULT_PROTOCOL,            i, 0, OPT_NOT_SET);
+      opt_i(&(opt.protocol), DEFAULT_PROTOCOL, &i, 0, OPT_NOT_SET);
     }
-    opt_i(opt.no_daemon,       DEFAULT_NO_DAEMON,           i, 0, OPT_NOT_SET);
-    opt_i(opt.no_lookup,       DEFAULT_NO_LOOKUP,           i, 0, OPT_NOT_SET);
-    opt_i(opt.login_disabled,  DEFAULT_LOGIN_DISABLED,      i, 0, OPT_NOT_SET);
-    opt_i(opt.lower_case,      DEFAULT_LOWER_CASE,          i, 0, OPT_NOT_SET);
-    opt_i(opt.server_resp_line,DEFAULT_SERVER_RESP_LINE,    i, 0, OPT_NOT_SET);
-    opt_i(opt.strip_domain,    DEFAULT_STRIP_DOMAIN,        i, 0, OPT_NOT_SET);
-    opt_i(opt.timeout,         DEFAULT_TIMEOUT,             i, 0, OPT_NOT_SET);
-    opt_i(opt.username_from_database, DEFAULT_USERNAME_FROM_DATABASE, 
-                                                            i, 0, OPT_NOT_SET);
-    opt_i(opt.quiet,           DEFAULT_QUIET,               i, 0, OPT_NOT_SET);
-    opt_i(opt.connect_relog,   DEFAULT_CONNECT_RELOG,       i, 0, OPT_NOT_SET);
-    opt_p(opt.capability,      PERDITION_PROTOCOL_DEPENDANT,i, 0, OPT_NOT_SET);
-    opt_p(opt.mangled_capability, NULL,                     i, 0, OPT_NOT_SET);
-    opt_p(opt.bind_address,    DEFAULT_BIND_ADDRESS,        i, 0, OPT_NOT_SET);
-    opt_p(opt.log_facility,    DEFAULT_LOG_FACILITY,        i, 0, OPT_NOT_SET);
+    opt_i(&(opt.no_daemon), DEFAULT_NO_DAEMON, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.no_lookup), DEFAULT_NO_LOOKUP, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.login_disabled), DEFAULT_LOGIN_DISABLED, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.lower_case), DEFAULT_LOWER_CASE, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.server_resp_line), DEFAULT_SERVER_RESP_LINE, 
+		    &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.strip_domain), DEFAULT_STRIP_DOMAIN, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.timeout), DEFAULT_TIMEOUT, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.username_from_database), DEFAULT_USERNAME_FROM_DATABASE, 
+   		    &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.quiet), DEFAULT_QUIET, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.connect_relog), DEFAULT_CONNECT_RELOG, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.capability), PERDITION_PROTOCOL_DEPENDANT, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.mangled_capability), NULL, &i, 0, OPT_NOT_SET);
+    opt_da(&(opt.bind_address), DEFAULT_BIND_ADDRESS, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.log_facility), DEFAULT_LOG_FACILITY, &i, 0, OPT_NOT_SET);
     if(!(f&OPT_FILE)) {
       char *filename;
       filename = config_file_name(basename, opt.protocol);
-      opt_p(opt.config_file,   filename,                    i, 0, OPT_NOT_SET);
+      opt_p(&(opt.config_file), filename, &i, 0, OPT_NOT_SET);
     }
-    opt_p(opt.domain_delimiter,DEFAULT_DOMAIN_DELIMITER,    i, 0, OPT_NOT_SET);
-    opt_p(opt.explicit_domain, NULL,                        i, 0, OPT_NOT_SET);
-    opt_p(opt.group,           DEFAULT_GROUP,               i, 0, OPT_NOT_SET);
-    opt_p(opt.listen_port,     PERDITION_PROTOCOL_DEPENDANT,i, 0, OPT_NOT_SET);
-    opt_p(opt.map_library,     DEFAULT_MAP_LIB,             i, 0, OPT_NOT_SET);
-    opt_p(opt.map_library_opt, DEFAULT_MAP_LIB_OPT,         i, 0, OPT_NOT_SET);
-    opt_p(opt.outgoing_port,   PERDITION_PROTOCOL_DEPENDANT,i, 0, OPT_NOT_SET);
-    opt_p(opt.ok_line,         DEFAULT_OK_LINE             ,i, 0, OPT_NOT_SET);
+    opt_p(&(opt.domain_delimiter), DEFAULT_DOMAIN_DELIMITER, 
+		    &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.explicit_domain), NULL, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.group), DEFAULT_GROUP, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.listen_port), PERDITION_PROTOCOL_DEPENDANT, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.log_passwd), DEFAILT_LOG_PASSWD, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.map_library), DEFAULT_MAP_LIB, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.map_library_opt), DEFAULT_MAP_LIB_OPT, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.outgoing_port), PERDITION_PROTOCOL_DEPENDANT, 
+		    &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.ok_line), DEFAULT_OK_LINE, &i, 0, OPT_NOT_SET);
 
-    opt_p(opt.username,        DEFAULT_USERNAME,            i, 0, OPT_NOT_SET);
-    opt_da(opt.outgoing_server,DEFAULT_OUTGOING_SERVER,     i, 0, OPT_NOT_SET);
-    opt_p(opt.pid_file,        PERDITION_PROTOCOL_DEPENDANT,i, 0, OPT_NOT_SET);
+    opt_p(&(opt.username), DEFAULT_USERNAME, &i, 0, OPT_NOT_SET);
+    opt_da(&(opt.outgoing_server), DEFAULT_OUTGOING_SERVER, 
+		    &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.pid_file), PERDITION_PROTOCOL_DEPENDANT,&i, 0, OPT_NOT_SET);
     {
       char *filename;
       filename = malloc(strlen(PERDITION_PID_DIR) + (2*strlen(basename)) + 7);
@@ -416,35 +522,35 @@ int options(int argc, char **argv, flag_t f){
       }
       sprintf(filename, "%s/%s/%s.pid", PERDITION_PID_DIR, basename,
 		      basename);
-      opt_p(opt.pid_file,        filename,                  i, 0, OPT_NOT_SET);
+      opt_p(&(opt.pid_file), filename, &i, 0, OPT_NOT_SET);
       free(filename);
     }
-    opt_da(opt.query_key,      DEFAULT_QUERY_KEY,           i, 0, OPT_NOT_SET);
+    opt_da(&(opt.query_key), DEFAULT_QUERY_KEY, &i, 0, OPT_NOT_SET);
 #ifdef WITH_SSL_SUPPORT
-    opt_i(opt.ssl_mode,        DEFAULT_SSL_MODE,            i, 0, OPT_NOT_SET);
-    opt_p(opt.ssl_ca_file,     DEFAULT_SSL_CA_FILE,         i, 0, OPT_NOT_SET);
-    opt_p(opt.ssl_ca_path,     DEFAULT_SSL_CA_PATH,         i, 0, OPT_NOT_SET);
-    opt_i(opt.ssl_ca_accept_self_signed, DEFAULT_SSL_CA_ACCEPT_SELF_SIGNED, 
-		                                            i, 0, OPT_NOT_SET);
-    opt_p(opt.ssl_cert_file,   DEFAULT_SSL_CERT_FILE,       i, 0, OPT_NOT_SET);
-    opt_i(opt.ssl_cert_accept_expired, DEFAULT_SSL_CERT_ACCEPT_EXPIRED, 
-		                                            i, 0, OPT_NOT_SET);
-    opt_i(opt.ssl_cert_accept_not_yet_valid, 
-		               DEFAULT_SSL_CERT_ACCEPT_NOT_YET_VALID, 
-		                                            i, 0, OPT_NOT_SET);
-    opt_i(opt.ssl_cert_accept_self_signed, DEFAULT_SSL_CERT_ACCEPT_SELF_SIGNED, 
-		                                            i, 0, OPT_NOT_SET);
-    opt_i(opt.ssl_cert_verify_depth, DEFAULT_SSL_CERT_VERIFY_DEPTH,
-		                                            i, 0, OPT_NOT_SET);
-    opt_p(opt.ssl_key_file,    DEFAULT_SSL_KEY_FILE,        i, 0, OPT_NOT_SET);
-    opt_p(opt.ssl_ca_chain_file, DEFAULT_SSL_CA_CHAIN_FILE, i, 0, OPT_NOT_SET);
-    opt_p(opt.ssl_listen_ciphers, DEFAULT_SSL_LISTEN_CIPHERS,
-		                                            i, 0, OPT_NOT_SET);
-    opt_p(opt.ssl_outgoing_ciphers, DEFAULT_SSL_OUTGOING_CIPHERS,
-		                                            i, 0, OPT_NOT_SET);
-    opt_i(opt.ssl_no_cert_verify, DEFAULT_SSL_NO_CERT_VERIFY,
-		                                            i, 0, OPT_NOT_SET);
-    opt_i(opt.ssl_no_cn_verify,DEFAULT_SSL_NO_CN_VERIFY,    i, 0, OPT_NOT_SET);
+    opt_i(&(opt.ssl_mode), DEFAULT_SSL_MODE, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.ssl_ca_file), DEFAULT_SSL_CA_FILE, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.ssl_ca_path), DEFAULT_SSL_CA_PATH, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.ssl_ca_accept_self_signed), DEFAULT_SSL_CA_ACCEPT_SELF_SIGNED, 
+		    &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.ssl_cert_file), DEFAULT_SSL_CERT_FILE, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.ssl_cert_accept_expired), DEFAULT_SSL_CERT_ACCEPT_EXPIRED, 
+	       	    &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.ssl_cert_accept_not_yet_valid), 
+		    DEFAULT_SSL_CERT_ACCEPT_NOT_YET_VALID, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.ssl_cert_accept_self_signed), 
+		    DEFAULT_SSL_CERT_ACCEPT_SELF_SIGNED, &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.ssl_cert_verify_depth), DEFAULT_SSL_CERT_VERIFY_DEPTH,
+		    &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.ssl_key_file), DEFAULT_SSL_KEY_FILE, &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.ssl_ca_chain_file), DEFAULT_SSL_CA_CHAIN_FILE,
+		    &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.ssl_listen_ciphers), DEFAULT_SSL_LISTEN_CIPHERS,
+		    &i, 0, OPT_NOT_SET);
+    opt_p(&(opt.ssl_outgoing_ciphers), DEFAULT_SSL_OUTGOING_CIPHERS,
+		    &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.ssl_no_cert_verify), DEFAULT_SSL_NO_CERT_VERIFY,
+		    &i, 0, OPT_NOT_SET);
+    opt_i(&(opt.ssl_no_cn_verify),DEFAULT_SSL_NO_CN_VERIFY, &i, 0, OPT_NOT_SET);
 #endif /* WITH_SSL_SUPPORT */
   }
 
@@ -472,7 +578,7 @@ int options(int argc, char **argv, flag_t f){
         break;
       case 'a':
 #ifdef WITH_PAM_SUPPORT
-        opt_i(opt.authenticate_in,1,opt.mask,MASK_AUTHENTICATE_IN,f); \
+        opt_i(&(opt.authenticate_in), 1, &(opt.mask), MASK_AUTHENTICATE_IN, f);
         break;
 #else
       VANESSA_LOGGER_DEBUG_RAW(
@@ -484,50 +590,63 @@ int options(int argc, char **argv, flag_t f){
         poptFreeContext(context);
         return(-1);
       }
-#endif /* WITH_PAM_SUPPORT */
+#endif
+      case TAG_AUTHENTICATE_TIMEOUT:
+        if(!vanessa_socket_str_is_digit(optarg) && f&OPT_ERR) 
+	  usage(-1);
+        opt_i(&(opt.authenticate_timeout), atoi(optarg), &(opt.mask2),
+	      MASK2_AUTHENTICATE_TIMEOUT, f);
+        break;
       case 'B':
-        opt_i(opt.no_bind_banner,1,opt.mask,MASK_NO_BIND_BANNER,f);
+        opt_i(&(opt.no_bind_banner), 1, &(opt.mask), MASK_NO_BIND_BANNER, f);
         break;
       case 'b':
-        opt_p(opt.bind_address,optarg,opt.mask,MASK_BIND_ADDRESS,f);
+        if(options_set_mask(&(opt.mask), f, MASK_BIND_ADDRESS)){
+          if(!(f&OPT_NOT_SET) && opt.bind_address!=NULL) {
+            vanessa_dynamic_array_destroy(opt.bind_address);
+          }
+          OPTARG_DUP;
+          opt.bind_address=split_str_server_port(
+            optarg_copy,
+            OPT_SERVER_DELIMITER
+          );
+        }
         break;
       case TAG_CONNECT_RELOG:
         if(!vanessa_socket_str_is_digit(optarg) && f&OPT_ERR){ 
 	  usage(-1); 
 	}
-        opt_i(opt.connect_relog,atoi(optarg),opt.mask,MASK_CONNECT_RELOG,f);
+        opt_i(&(opt.connect_relog), atoi(optarg), &(opt.mask), 
+			MASK_CONNECT_RELOG, f);
         break;
       case 'C':
-        opt_i(opt.connection_logging,1,opt.mask,MASK_DEBUG,f);
+        opt_i(&(opt.connection_logging), 1, &(opt.mask), MASK_DEBUG, f);
         break;
       case 'c':
-        opt_i(
-          opt.client_server_specification,
-          1,
-          opt.mask,
-          MASK_CLIENT_SERVER_SPECIFICATION,
-          f
-        );
+        opt_i(&(opt.client_server_specification), 1, &(opt.mask),
+			MASK_CLIENT_SERVER_SPECIFICATION, f);
         break;
       case 'D':
-        opt_p(opt.domain_delimiter,optarg,opt.mask,MASK_DOMAIN_DELIMITER,f);
+        opt_p(&(opt.domain_delimiter), optarg, &(opt.mask), 
+			MASK_DOMAIN_DELIMITER, f);
         break;
       case 'd':
-        opt_i(opt.debug,1,opt.mask,MASK_DEBUG,f);
+        opt_i(&(opt.debug), 1, &(opt.mask), MASK_DEBUG, f);
         break;
       case 'e':
-	opt_p(opt.explicit_domain,optarg,opt.mask2,MASK2_EXPLICIT_DOMAIN,f);
+	opt_p(&(opt.explicit_domain), optarg, &(opt.mask2),
+			MASK2_EXPLICIT_DOMAIN, f);
 	break;
       case 'f':
         if(!(f&OPT_FILE)){
-          opt_p(opt.config_file,optarg,opt.mask,MASK_CONFIG_FILE,f);
+          opt_p(&(opt.config_file), optarg, &(opt.mask), MASK_CONFIG_FILE, f);
         }
         break;
       case 'F':
-        opt_p(opt.log_facility,optarg,opt.mask,MASK_LOG_FACILITY,f);
+        opt_p(&(opt.log_facility), optarg, &(opt.mask), MASK_LOG_FACILITY, f);
         break;
       case 'g':
-        opt_p(opt.group,optarg,opt.mask,MASK_GROUP,f);
+        opt_p(&(opt.group), optarg, &(opt.mask), MASK_GROUP, f);
         break;
       case 'h':
         if(f&OPT_ERR && !(f&OPT_FILE)){ 
@@ -535,10 +654,10 @@ int options(int argc, char **argv, flag_t f){
 	}
 	break;
       case 'I':
-	opt_p(opt.capability,optarg,opt.mask,MASK_CAPABILITY,f);
+	opt_p(&(opt.capability), optarg, &(opt.mask), MASK_CAPABILITY, f);
 	break;
       case 'i':
-        opt_i(opt.inetd_mode,1,opt.mask,MASK_INETD_MODE,f);
+        opt_i(&(opt.inetd_mode), 1, &(opt.mask), MASK_INETD_MODE, f);
         break;
       case 'j':
 	VANESSA_LOGGER_DEBUG( "Jain, Oath\n"); 
@@ -547,31 +666,27 @@ int options(int argc, char **argv, flag_t f){
         if(!vanessa_socket_str_is_digit(optarg) && f&OPT_ERR){ 
 	  usage(-1); 
 	}
-        opt_i(
-	  opt.connection_limit,
-	  atoi(optarg),
-	  opt.mask,
-	  MASK_CONNECTION_LIMIT,
-	  f
-	); \
+        opt_i(&(opt.connection_limit), atoi(optarg), &(opt.mask),
+			MASK_CONNECTION_LIMIT, f);
         break;
       case 'l':
-        opt_p(opt.listen_port,optarg,opt.mask,MASK_LISTEN_PORT,f);
+        opt_p(&(opt.listen_port), optarg, &(opt.mask), MASK_LISTEN_PORT, f);
         break;
       case 'M':
-        opt_p(opt.map_library,optarg,opt.mask,MASK_MAP_LIB,f);
+        opt_p(&(opt.map_library), optarg, &(opt.mask), MASK_MAP_LIB, f);
         break;
       case 'm':
-        opt_p(opt.map_library_opt,optarg,opt.mask,MASK_MAP_LIB_OPT,f);
+        opt_p(&(opt.map_library_opt), optarg, &(opt.mask), MASK_MAP_LIB_OPT, f);
         break;
       case 'n':
-        opt_i(opt.no_lookup,1,opt.mask,MASK_NO_LOOKUP,f);
+        opt_i(&(opt.no_lookup), 1, &(opt.mask), MASK_NO_LOOKUP, f);
         break;
       case 'O':
-        opt_p(opt.ok_line,optarg,opt.mask2,MASK2_OK_LINE,f);
+        opt_p(&(opt.ok_line), optarg, &(opt.mask2), MASK2_OK_LINE, f);
         break;
       case 'o':
-        opt_i(opt.server_resp_line,1,opt.mask,MASK_SERVER_RESP_LINE,f);
+        opt_i(&(opt.server_resp_line), 1, &(opt.mask),
+			MASK_SERVER_RESP_LINE, f);
         break;
       case 'P':
         if((index=protocol_index(optarg))<0){
@@ -579,11 +694,11 @@ int options(int argc, char **argv, flag_t f){
 		usage(-1);
         }
         else {
-          opt_i(opt.protocol,index,opt.mask,MASK_PROTOCOL,f);
+          opt_i(&(opt.protocol), index, &(opt.mask), MASK_PROTOCOL, f);
         }
         break;
       case 'p':
-        opt_p(opt.outgoing_port,optarg,opt.mask,MASK_OUTGOING_PORT,f);
+        opt_p(&(opt.outgoing_port), optarg, &(opt.mask), MASK_OUTGOING_PORT, f);
         break;
       case 'S':
         OPTARG_DUP;
@@ -610,23 +725,23 @@ int options(int argc, char **argv, flag_t f){
         if(!vanessa_socket_str_is_digit(optarg) && f&OPT_ERR){ 
 	  usage(-1); 
 	}
-        opt_i(opt.timeout,atoi(optarg),opt.mask,MASK_TIMEOUT,f);
+        opt_i(&(opt.timeout), atoi(optarg), &(opt.mask), MASK_TIMEOUT, f);
         break;
       case 'u':
-        opt_p(opt.username,optarg,opt.mask,MASK_USERNAME,f);
+        opt_p(&(opt.username), optarg, &(opt.mask), MASK_USERNAME, f);
         break;
       case 'U':
-        opt_i(opt.username_from_database,1,opt.mask,
-	  MASK_USERNAME_FROM_DATABASE,f);
+        opt_i(&(opt.username_from_database), 1, &(opt.mask), 
+			MASK_USERNAME_FROM_DATABASE, f);
         break;
       case 'q':
-        opt_i(opt.quiet,1,opt.mask,MASK_QUIET,f);
+        opt_i(&(opt.quiet), 1, &(opt.mask), MASK_QUIET, f);
         break;
       case TAG_NO_DAEMON:
-	opt_i(opt.no_daemon,1,opt.mask,MASK_NO_DAEMON,f);
+	opt_i(&(opt.no_daemon), 1, &(opt.mask), MASK_NO_DAEMON, f);
         break;
       case TAG_LOGIN_DISABLED:
-	opt_i(opt.login_disabled,1,opt.mask,MASK_LOGIN_DISABLED,f);
+	opt_i(&(opt.login_disabled), 1, &(opt.mask), MASK_LOGIN_DISABLED, f);
         break;
       case TAG_LOWER_CASE:
         OPTARG_DUP;
@@ -654,8 +769,17 @@ int options(int argc, char **argv, flag_t f){
 	  VANESSA_LOGGER_ERR_UNSAFE("Invalid pid file: \"%s\"", optarg);
 	  usage(-1);
 	}
-        opt_p(opt.pid_file,optarg,opt.mask2,MASK2_PID_FILE,f);
+        opt_p(&(opt.pid_file), optarg, &(opt.mask2), MASK2_PID_FILE, f);
         break;
+      case TAG_LOG_PASSWD:
+       	opt_i(&(opt.log_passwd), log_passwd_from_str(optarg), 
+			&(opt.mask2), MASK2_LOG_PASSWD, f);
+	if (opt.log_passwd < 0) {
+	       	VANESSA_LOGGER_ERR_UNSAFE("Invalid log_passwd mode: \"%s\"", 
+				optarg);
+		usage(-1);
+	}
+	break;
       case TAG_SSL_MODE:
 #ifdef WITH_SSL_SUPPORT
         OPTARG_DUP;
@@ -671,38 +795,38 @@ int options(int argc, char **argv, flag_t f){
       break;
       case TAG_SSL_CA_FILE:
 #ifdef WITH_SSL_SUPPORT
-        opt_p(opt.ssl_ca_file,optarg,opt.ssl_mask,MASK_SSL_CA_FILE,f);
+        opt_p(&(opt.ssl_ca_file), optarg, &(opt.ssl_mask), MASK_SSL_CA_FILE, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_ca_file");
 #endif /* WITH_SSL_SUPPORT */
       break;
       case TAG_SSL_CA_PATH:
 #ifdef WITH_SSL_SUPPORT
-        opt_p(opt.ssl_ca_path,optarg,opt.ssl_mask,MASK_SSL_CA_PATH,f);
+        opt_p(&(opt.ssl_ca_path), optarg, &(opt.ssl_mask), MASK_SSL_CA_PATH, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_ca_path");
 #endif /* WITH_SSL_SUPPORT */
       break;
       case TAG_SSL_CA_ACCEPT_SELF_SIGNED:
 #ifdef WITH_SSL_SUPPORT
-        opt_i(opt.ssl_ca_accept_self_signed,1,opt.ssl_mask,
-			MASK_SSL_CA_ACCEPT_SELF_SIGNED,f);
+        opt_i(&(opt.ssl_ca_accept_self_signed), 1, &(opt.ssl_mask),
+			MASK_SSL_CA_ACCEPT_SELF_SIGNED, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_ca_accept_self_signed");
 #endif /* WITH_SSL_SUPPORT */
       break;
       case TAG_SSL_CERT_VERIFY_DEPTH:
 #ifdef WITH_SSL_SUPPORT
-        opt_i(opt.ssl_cert_verify_depth,atoi(optarg),opt.ssl_mask,
-			MASK_SSL_CERT_VERIFY_DEPTH,f);
+        opt_i(&(opt.ssl_cert_verify_depth), atoi(optarg), &(opt.ssl_mask),
+			MASK_SSL_CERT_VERIFY_DEPTH, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_ca_verify_depth");
 #endif /* WITH_SSL_SUPPORT */
       break;
       case TAG_SSL_CA_CHAIN_FILE:
 #ifdef WITH_SSL_SUPPORT
-        opt_p(opt.ssl_ca_chain_file,optarg,opt.ssl_mask,
-			MASK_SSL_CA_CHAIN_FILE,f);
+        opt_p(&(opt.ssl_ca_chain_file), optarg, &(opt.ssl_mask),
+			MASK_SSL_CA_CHAIN_FILE, f);
 #else /* WITH_SSL_SUPPORT */
       VANESSA_LOGGER_DEBUG(
 	"--ssl_ca_chain_file is only supported when ssl support "
@@ -718,45 +842,47 @@ int options(int argc, char **argv, flag_t f){
         break; 
       case TAG_SSL_CERT_FILE:
 #ifdef WITH_SSL_SUPPORT
-        opt_p(opt.ssl_cert_file,optarg,opt.ssl_mask,MASK_SSL_CERT_FILE,f);
+        opt_p(&(opt.ssl_cert_file), optarg, &(opt.ssl_mask), 
+			MASK_SSL_CERT_FILE, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_cert_file");
 #endif /* WITH_SSL_SUPPORT */
         break; 
       case TAG_SSL_CERT_ACCEPT_EXPIRED:
 #ifdef WITH_SSL_SUPPORT
-        opt_i(opt.ssl_cert_accept_expired,1,opt.ssl_mask,
-			MASK_SSL_CERT_ACCEPT_EXPIRED,f);
+        opt_i(&(opt.ssl_cert_accept_expired), 1, &(opt.ssl_mask),
+			MASK_SSL_CERT_ACCEPT_EXPIRED, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_cert_accept_expired");
 #endif /* WITH_SSL_SUPPORT */
       break;
       case TAG_SSL_CERT_ACCEPT_NOT_YET_VALID:
 #ifdef WITH_SSL_SUPPORT
-        opt_i(opt.ssl_cert_accept_not_yet_valid,1,opt.ssl_mask,
-			MASK_SSL_CERT_ACCEPT_NOT_YET_VALID,f);
+        opt_i(&(opt.ssl_cert_accept_not_yet_valid), 1, &(opt.ssl_mask),
+			MASK_SSL_CERT_ACCEPT_NOT_YET_VALID, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_cert_accept_not_yet_valid");
 #endif /* WITH_SSL_SUPPORT */
       break;
       case TAG_SSL_CERT_ACCEPT_SELF_SIGNED:
 #ifdef WITH_SSL_SUPPORT
-        opt_i(opt.ssl_cert_accept_self_signed,1,opt.ssl_mask,
-			MASK_SSL_CERT_ACCEPT_SELF_SIGNED,f);
+        opt_i(&(opt.ssl_cert_accept_self_signed), 1, &(opt.ssl_mask),
+			MASK_SSL_CERT_ACCEPT_SELF_SIGNED, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_cert_accept_self_signed");
 #endif /* WITH_SSL_SUPPORT */
       break;
       case TAG_SSL_KEY_FILE:
 #ifdef WITH_SSL_SUPPORT
-        opt_p(opt.ssl_key_file,optarg,opt.ssl_mask,MASK_SSL_KEY_FILE,f);
+        opt_p(&(opt.ssl_key_file), optarg, &(opt.ssl_mask), 
+			MASK_SSL_KEY_FILE, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_key_file");
 #endif /* WITH_SSL_SUPPORT */
         break; 
       case TAG_SSL_LISTEN_CIPHERS:
 #ifdef WITH_SSL_SUPPORT
-        opt_p(opt.ssl_listen_ciphers,optarg,opt.ssl_mask,
+        opt_p(&(opt.ssl_listen_ciphers), optarg, &(opt.ssl_mask),
 			MASK_SSL_LISTEN_CIPHERS,f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_listen_ciphers");
@@ -764,22 +890,24 @@ int options(int argc, char **argv, flag_t f){
         break; 
       case TAG_SSL_OUTGOING_CIPHERS:
 #ifdef WITH_SSL_SUPPORT
-        opt_p(opt.ssl_outgoing_ciphers,optarg,opt.ssl_mask,
-			MASK_SSL_OUTGOING_CIPHERS,f);
+        opt_p(&(opt.ssl_outgoing_ciphers), optarg, &(opt.ssl_mask),
+			MASK_SSL_OUTGOING_CIPHERS, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_outgoing_ciphers");
 #endif /* WITH_SSL_SUPPORT */
         break; 
       case TAG_SSL_NO_CERT_VERIFY:
 #ifdef WITH_SSL_SUPPORT
-        opt_i(opt.ssl_no_cert_verify,1,opt.ssl_mask, MASK_SSL_NO_CERT_VERIFY,f);
+        opt_i(&(opt.ssl_no_cert_verify), 1, &(opt.ssl_mask), 
+			MASK_SSL_NO_CERT_VERIFY, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_no_cert_verify");
 #endif /* WITH_SSL_SUPPORT */
         break; 
       case TAG_SSL_NO_CN_VERIFY:
 #ifdef WITH_SSL_SUPPORT
-        opt_i(opt.ssl_no_cn_verify,1,opt.ssl_mask, MASK_SSL_NO_CN_VERIFY,f);
+        opt_i(&(opt.ssl_no_cn_verify), 1, &(opt.ssl_mask),
+			MASK_SSL_NO_CN_VERIFY, f);
 #else /* WITH_SSL_SUPPORT */
 	NO_SSL_OPT("ssl_no_cn_verify");
 #endif /* WITH_SSL_SUPPORT */
@@ -819,24 +947,6 @@ int options(int argc, char **argv, flag_t f){
 }
 
 
-/**********************************************************************
- * options_set_mask
- * Set the options mask
- * pre: mask: pointer to current mask that may be modified
- *      mask_entry: value to or with opt->mask
- *      flag: flags
- * post: mask is added if flags permit
- * return: 1 if mask is added
- *         0 otherwise
- **********************************************************************/
-
-int options_set_mask(flag_t *mask, flag_t mask_entry, flag_t flag){
-  if(flag&OPT_USE_MASK && (*mask)&mask_entry) return(0);
-  if(flag&OPT_SET_MASK) (*mask)|=mask_entry;
-  return(1);
-}
-
-
 #define BIN_OPT_STR(_opt) ((_opt)?"on":"off")
 #define OPT_STR(_opt) ((_opt)?(_opt):"")
 
@@ -885,22 +995,20 @@ int options_set_mask(flag_t *mask, flag_t mask_entry, flag_t flag){
   } \
 }
 
-
 int log_options_str(char *str, size_t n){
+  int status = -1;
   char *protocol=NULL;
   char *outgoing_server=NULL;
+  char *bind_address=NULL;
   char *query_key=NULL;
   char add_domain[40];
   char lower_case[40];
   char strip_domain[40];
 #ifdef WITH_SSL_SUPPORT
-  char ssl_mode[26];
+  char ssl_mode[40];
   char *ssl_mode_p = NULL;
 #endif /* WITH_SSL_SUPPORT */
 
-  extern options_t opt;
-  extern struct utsname *system_uname;
-
   if((protocol=protocol_list(protocol, NULL, opt.protocol))==NULL){
     VANESSA_LOGGER_DEBUG("protocol_list");
     return(-1);
@@ -912,8 +1020,17 @@ int log_options_str(char *str, size_t n){
       OPT_SERVER_DELIMITER
     ))==NULL){
       VANESSA_LOGGER_DEBUG("vanessa_dynamic_array_display");
-      free(protocol);
-      return(-1);
+      goto err;
+    }
+  }
+
+  if(opt.bind_address!=NULL){
+    if((bind_address=vanessa_dynamic_array_display(
+      opt.bind_address,
+      OPT_SERVER_DELIMITER
+    ))==NULL){
+      VANESSA_LOGGER_DEBUG("vanessa_dynamic_array_display");
+      goto err;
     }
   }
 
@@ -923,9 +1040,7 @@ int log_options_str(char *str, size_t n){
       OPT_SERVER_DELIMITER
     ))==NULL){
       VANESSA_LOGGER_DEBUG("vanessa_dynamic_array_display");
-      free(outgoing_server);
-      free(protocol);
-      return(-1);
+      goto err;
     }
   }
 
@@ -989,6 +1104,7 @@ int log_options_str(char *str, size_t n){
 #ifdef WITH_PAM_SUPPORT
     "authenticate_in=%s, "
 #endif /* WITH_PAM_SUPPORT */
+    "authenticate_timeout=%d, "
     "bind_address=\"%s\", "
     "capability=\"%s\", "
     "client_server_specification=%s, "
@@ -1003,6 +1119,7 @@ int log_options_str(char *str, size_t n){
     "inetd_mode=%s, "
     "listen_port=\"%s\", "
     "log_facility=\"%s\", "
+    "log_passwd=\"%s\", "
     "login_disabled=%s, "
     "lower_case=\"%s\", "
     "map_library=\"%s\", "
@@ -1045,7 +1162,8 @@ int log_options_str(char *str, size_t n){
 #ifdef WITH_PAM_SUPPORT
     BIN_OPT_STR(opt.authenticate_in),
 #endif /* WITH_PAM_SUPPORT */
-    OPT_STR(opt.bind_address),
+    opt.authenticate_timeout,
+    OPT_STR(bind_address),
     OPT_STR(opt.capability),
     BIN_OPT_STR(opt.client_server_specification),
     OPT_STR(opt.config_file),
@@ -1059,6 +1177,7 @@ int log_options_str(char *str, size_t n){
     BIN_OPT_STR(opt.inetd_mode),
     OPT_STR(opt.listen_port),
     OPT_STR(opt.log_facility),
+    OPT_STR(log_passwd_to_str(opt.log_passwd)),
     BIN_OPT_STR(opt.login_disabled),
     OPT_STR(lower_case),
     OPT_STR(opt.map_library),
@@ -1100,10 +1219,13 @@ int log_options_str(char *str, size_t n){
   );
   *(str+n-1) = '\0';
 
+  status = 0;
+err:
   str_free(protocol);
   str_free(outgoing_server);
+  str_free(bind_address);
   str_free(query_key);
-  return(0);
+  return status;
 }
 
 
@@ -1188,10 +1310,13 @@ void usage(int exit_status){
     " -a|--authenticate_in:\n"
     "    User is authenticated by perdition before connection to real-server.\n"
 #endif /* WITH_PAM_SUPPORT */
+    " --authenticate_timeout:\n"
+    "    Timeout used during authentication phase.\n"
+    "    Zero for infinite timeout. (default %d)\n"
     " -B|--no_bind_banner:\n"
     "    Use uname to generate banner of even if bind_address is in effect.\n"
-    " -b|--bind_address IP_ADDRESS|HOSTNAME:\n"
-    "    Bind to this address.\n"
+    " -b|--bind_address SERVER[,SERVER...]:\n"
+    "    Bind to these addresses and ports.\n"
     "    (default \"%s\")\n"
     " -C|--connection_logging:\n"
     "    Log all communication received from end-users or real servers or\n"
@@ -1230,6 +1355,9 @@ void usage(int exit_status){
     "    Port to listen on. (default \"%s\")\n"
     " --login_disabled:\n"
     "    Do not allow users to log in.\n"
+    " --log_passwd STATE:\n"
+    "    Log the users password, otherwise just report it as \"XXXXXX\".\n"
+    "    (default \"%s\")\n"
     " --lower_case STATE[,STATE...]:\n"
     "    Convert usernames to lower case according the the locale in given\n"
     "    state(s). (default \"\")\n"
@@ -1266,7 +1394,8 @@ void usage(int exit_status){
     "    Allow domain portion of username to be striped in given state(s).\n"
     "    (default \"\")\n"
     " -t|--timeout SECONDS:\n"
-    "    Idle timeout. Zero for infinite timeout. (default %d)\n"
+    "    Idle timeout for post-authentication phase.\n"
+    "    Zero for infinite timeout. (default %d)\n"
     " -u|--username USERNAME:\n"
     "    User to run as. (default \"%s\")\n"
     " -U|--username_from_database:\n"
@@ -1338,6 +1467,7 @@ void usage(int exit_status){
     "        be used unless noted otherwise.\n"
     "        See the perdition(8) man page for more details.\n",
     VERSION,
+    DEFAULT_AUTHENTICATE_TIMEOUT,
     OPT_STR(DEFAULT_BIND_ADDRESS),
     DEFAULT_CONNECT_RELOG,
     OPT_STR(DEFAULT_DOMAIN_DELIMITER),
@@ -1346,6 +1476,7 @@ void usage(int exit_status){
     OPT_STR(PERDITION_PROTOCOL_DEPENDANT),
     DEFAULT_CONNECTION_LIMIT,
     OPT_STR(PERDITION_PROTOCOL_DEPENDANT),
+    OPT_STR(log_passwd_to_str(DEFAILT_LOG_PASSWD)),
     OPT_STR(DEFAULT_MAP_LIB),
     OPT_STR(DEFAULT_MAP_LIB_OPT),
     OPT_STR(DEFAULT_OK_LINE),
