@@ -18,11 +18,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -35,7 +34,6 @@
 #include <sys/types.h>   /*For u_int32_t */
 #include <vanessa_adt.h>
 #include <popt.h>
-#include <jain.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -98,6 +96,15 @@
 #define SSL_SSL_MASK           SSL_MODE_SSL_ALL
 #define SSL_TLS_MASK           SSL_MODE_TLS_ALL
 
+#define LOG_PASSWD_NEVER       0x00
+#define LOG_PASSWD_NEVER_STR   "never"
+#define LOG_PASSWD_FAIL        0x01
+#define LOG_PASSWD_FAIL_STR    "fail"
+#define LOG_PASSWD_OK          0x02
+#define LOG_PASSWD_OK_STR      "ok"
+#define LOG_PASSWD_ALWAYS      (LOG_PASSWD_FAIL|LOG_PASSWD_OK)
+#define LOG_PASSWD_ALWAYS_STR  "always"
+
 
 #define DEFAULT_ADD_DOMAIN                   STATE_NONE
 #define DEFAULT_ADD_DOMAIN_STRIP_DEPTH       1
@@ -132,6 +139,7 @@
 #define DEFAULT_STRIP_DOMAIN                 STATE_NONE
 #define DEFAULT_SERVER_RESP_LINE             0
 #define DEFAULT_TIMEOUT                      1800 /*in seconds*/
+#define DEFAULT_AUTHENTICATE_TIMEOUT         DEFAULT_TIMEOUT
 #ifdef WITH_USER
 #define DEFAULT_USERNAME                     WITH_USER
 #else
@@ -140,6 +148,7 @@
 #define DEFAULT_USERNAME_FROM_DATABASE       0
 #define DEFAULT_QUERY_KEY                    NULL
 #define DEFAULT_QUIET                        0
+#define DEFAILT_LOG_PASSWD                   LOG_PASSWD_NEVER
 #ifdef WITH_SSL_SUPPORT
 #define DEFAULT_SSL_CA_CHAIN_FILE            NULL
 #define RECOMMENDED_SSL_CA_CHAIN_FILE         PERDITION_SYSCONFDIR \
@@ -172,7 +181,8 @@ typedef struct {
 #ifdef WITH_PAM_SUPPORT
   int             authenticate_in;
 #endif /* WITH_PAM_SUPPORT */
-  char            *bind_address;
+  int             authenticate_timeout;
+  vanessa_dynamic_array_t *bind_address;
   char            *capability;
   char            *mangled_capability;
   int             client_server_specification;
@@ -189,6 +199,7 @@ typedef struct {
   char            *pop_capability;
   char            *listen_port;
   char            *log_facility;
+  int             log_passwd;
   int             login_disabled;
   int             lower_case;
   char            *map_library;
@@ -267,6 +278,8 @@ typedef struct {
 #define MASK2_OK_LINE                    (flag_t) 0x00000001
 #define MASK2_PID_FILE                   (flag_t) 0x00000002
 #define MASK2_EXPLICIT_DOMAIN            (flag_t) 0x00000004
+#define MASK2_LOG_PASSWD                 (flag_t) 0x00000008
+#define MASK2_AUTHENTICATE_TIMEOUT       (flag_t) 0x00000010
 
 #ifdef WITH_SSL_SUPPORT
 /* options_t.ssl_mask entries */
@@ -297,23 +310,25 @@ typedef struct {
 #define TAG_LOGIN_DISABLED                     (int) 129
 #define TAG_LOWER_CASE                         (int) 130
 #define TAG_NO_DAEMON                          (int) 131
-#define TAG_PID_FILE                           (int) 132
-#define TAG_QUERY_KEY                          (int) 133
-#define TAG_SSL_CA_CHAIN_FILE                  (int) 134
-#define TAG_SSL_CA_FILE                        (int) 135
-#define TAG_SSL_CA_PATH                        (int) 136
-#define TAG_SSL_CA_ACCEPT_SELF_SIGNED          (int) 137
-#define TAG_SSL_CERT_FILE                      (int) 138
-#define TAG_SSL_CERT_ACCEPT_EXPIRED            (int) 139
-#define TAG_SSL_CERT_ACCEPT_SELF_SIGNED        (int) 140
-#define TAG_SSL_CERT_ACCEPT_NOT_YET_VALID      (int) 141
-#define TAG_SSL_CERT_VERIFY_DEPTH              (int) 142
-#define TAG_SSL_KEY_FILE                       (int) 143
-#define TAG_SSL_MODE                           (int) 144
-#define TAG_SSL_LISTEN_CIPHERS                 (int) 145
-#define TAG_SSL_OUTGOING_CIPHERS               (int) 146
-#define TAG_SSL_NO_CERT_VERIFY                 (int) 147
-#define TAG_SSL_NO_CN_VERIFY                   (int) 148
+#define TAG_QUERY_KEY                          (int) 132
+#define TAG_PID_FILE                           (int) 133
+#define TAG_LOG_PASSWD                         (int) 134
+#define TAG_SSL_CA_CHAIN_FILE                  (int) 135
+#define TAG_SSL_CA_FILE                        (int) 136
+#define TAG_SSL_CA_PATH                        (int) 137
+#define TAG_SSL_CA_ACCEPT_SELF_SIGNED          (int) 138
+#define TAG_SSL_CERT_FILE                      (int) 139
+#define TAG_SSL_CERT_ACCEPT_EXPIRED            (int) 140
+#define TAG_SSL_CERT_ACCEPT_SELF_SIGNED        (int) 141
+#define TAG_SSL_CERT_ACCEPT_NOT_YET_VALID      (int) 142
+#define TAG_SSL_CERT_VERIFY_DEPTH              (int) 143
+#define TAG_SSL_KEY_FILE                       (int) 144
+#define TAG_SSL_MODE                           (int) 145
+#define TAG_SSL_LISTEN_CIPHERS                 (int) 146
+#define TAG_SSL_OUTGOING_CIPHERS               (int) 147
+#define TAG_SSL_NO_CERT_VERIFY                 (int) 148
+#define TAG_SSL_NO_CN_VERIFY                   (int) 149
+#define TAG_AUTHENTICATE_TIMEOUT               (int) 150
 
 
 /*Flag values for options()*/
@@ -344,20 +359,6 @@ typedef struct {
 int options(int argc, char **argv, flag_t f);
 
 
-/**********************************************************************
- * options_set_mask
- * Set the options mask
- * pre: mask: pointer to current mask that may be modified
- *      mask_entry: value to or with opt->mask
- *      flag: flags
- * post: mask is added if flags permit
- * return: 1 if mask is added
- *         0 otherwise
- **********************************************************************/
-
-int options_set_mask(flag_t *mask, flag_t mask_entry, flag_t flag);
-
-
 /**********************************************************************
  * log_options_str
  * Log options to a string
