@@ -17,11 +17,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -58,8 +57,10 @@
 #include "server_port.h"
 #include "username.h"
 #include "ssl.h"
+#include "sock.h"
 #include "setproctitle.h"
 #include "imap4_tag.h"
+#include "perdition_globals.h"
 
 /* limits.h should be sufficient on most systems 
  * http://www.opengroup.org/onlinepubs/007908799/headix.html */
@@ -87,11 +88,11 @@ struct utsname *system_uname;
 /* Local and Peer address information is global so perditiondb
  * libraries can access this information
  */
-struct sockaddr_in *peername;
-struct sockaddr_in *sockname;
+struct sockaddr_storage *peername;
+struct sockaddr_storage *sockname;
 
 /* PID file that has been created */
-char *pid_file;
+static char *pid_file;
 
 /*
  * Used for opening dynamic server lookup library
@@ -158,36 +159,73 @@ static int perdition_chown(const char *path, const char *username,
     perdition_exit_cleanly(-1); \
   }
 
-/* Macro to log session just after Authentication */
-#define VANESSA_LOGGER_LOG_AUTH(_auth_log, _from_to_str,                   \
-			_user, _servername, _port, _status)                \
-	memset(_auth_log.log_str, 0, sizeof(_auth_log.log_str));           \
-	snprintf(_auth_log.log_str, sizeof(_auth_log.log_str)-1,           \
-			"Auth: %suser=\"%s\" server=\"%s\" port=\"%s\" "   \
-			"status=\"%s\"",                                   \
-			from_to_str, str_null_safe(_user),                 \
-			str_null_safe(_servername),                        \
-			str_null_safe(_port), str_null_safe(_status));     \
-	VANESSA_LOGGER_LOG(LOG_NOTICE, _auth_log.log_str);                 \
-	_auth_log.log_time=time(NULL) + opt.connect_relog;                 \
-	set_proc_title("%s: auth %s", progname, str_null_safe(_status));
+static void 
+perdition_log_auth(timed_log_t *auth_log, const char *from_to_host_str,
+		struct passwd *pw, const char *servername, const char *port, 
+		const char *progname, const char *reason)
+{
+	char *passwd;
 
-#define LOGIN_FAILED(_type, _reason)                                      \
-{                                                                         \
-	sleep(PERDITION_AUTH_FAIL_SLEEP);                                 \
-	if(protocol->write(client_io, NULL_FLAG, client_tag,              \
-				protocol->type[(_type)], 0,               \
-				(_reason))<0){                            \
-		VANESSA_LOGGER_DEBUG("protocol->write");                  \
-		VANESSA_LOGGER_ERR("Fatal error writing to client. "      \
-				"Exiting child.");                        \
-		perdition_exit_cleanly(-1);                   \
-	}                                                                 \
-	VANESSA_LOGGER_LOG_AUTH(auth_log, from_to_str, pw.pw_name,        \
-			servername, port,                                 \
-			"failed: " _reason);                              \
+	if (!strcmp(reason, "ok")) {
+		if (opt.log_passwd & LOG_PASSWD_OK)
+			passwd = pw->pw_passwd;
+		else 
+			passwd = "XXXXXX";
+	}
+	else {
+		if (opt.log_passwd & LOG_PASSWD_FAIL)
+			passwd = pw->pw_passwd;
+		else 
+			passwd = "XXXXXX";
+	}
+
+	memset(auth_log->log_str, 0, sizeof(auth_log->log_str));
+	snprintf(auth_log->log_str, sizeof(auth_log->log_str),
+			"Auth:%s user=\"%s\" passwd=\"%s\" "
+			"server=\"%s\" port=\"%s\" status=\"%s\"",
+			from_to_host_str, str_null_safe(pw->pw_name),
+			str_null_safe(passwd), str_null_safe(servername),
+			str_null_safe(port), str_null_safe(reason));
+	auth_log->log_time = time(NULL) + opt.connect_relog;
+
+	VANESSA_LOGGER_LOG(LOG_NOTICE, auth_log->log_str);
+
+	set_proc_title("%s: auth %s", progname, str_null_safe(reason));
 }
 
+#define PERDITION_LOG_AUTH(_reason)                                        \
+do {                                                                       \
+	perdition_log_auth(&auth_log, from_to_host_str, &pw, servername,   \
+			port, progname, _reason);                          \
+} while(0)
+
+static void 
+login_failed_protocol(protocol_t *protocol, int protocol_type, 
+		io_t *io, token_t *tag, timed_log_t *auth_log, 
+		const char *from_to_host_str, struct passwd *pw,
+		const char *servername, const char *port,
+		const char *progname, const char *reason)
+{
+	sleep(PERDITION_AUTH_FAIL_SLEEP);
+	if (protocol->write(io, NULL_FLAG, tag, protocol->type[protocol_type], 
+				0, reason) < 0) {
+		VANESSA_LOGGER_DEBUG("protocol->write");
+		VANESSA_LOGGER_ERR("Fatal error writing to client. "
+				"Exiting child.");
+		perdition_exit_cleanly(-1);
+	}
+
+	perdition_log_auth(auth_log, from_to_host_str, pw, servername, port,
+			progname, reason);
+}
+
+#define LOGIN_FAILED_PROTOCOL(_type, _reason)                               \
+do {                                                                        \
+	login_failed_protocol(protocol, _type, client_io, client_tag,       \
+			&auth_log, from_to_host_str, &pw, servername,       \
+			port, progname, "failed: " _reason);                \
+} while(0)
+
 /**********************************************************************
  * Muriel the main function
  **********************************************************************/
@@ -196,7 +234,6 @@ int main (int argc, char **argv, char **envp){
   vanessa_logger_t *vl;
   struct passwd pw = {NULL, NULL};
   struct passwd pw2 = {NULL, NULL};
-  struct in_addr *to_addr;
   unsigned char *server_resp_buf=NULL;
   unsigned char *buffer;
   user_server_port_t *usp=NULL;
@@ -206,9 +243,11 @@ int main (int argc, char **argv, char **envp){
   size_t server_resp_buf_size=0;
   flag_t tls_state=0;
   timed_log_t auth_log;
-  char from_to_str[36];
-  char from_str[17];
-  char to_str[17];
+  char from_to_host_str[(NI_MAXHOST*2)+2];
+  char from_host_str[NI_MAXHOST];
+  char to_host_str[NI_MAXHOST];
+  char from_serv_str[NI_MAXSERV];
+  char to_serv_str[NI_MAXSERV];
   char *servername=NULL;
   char *username=NULL;
   char *port=NULL;
@@ -222,17 +261,13 @@ int main (int argc, char **argv, char **envp){
   int round_robin_server=0;
   int rnd;
   int s=-1;
-  int g=-1;
+  int *g = NULL;
+  int rc;
 
 #ifdef WITH_SSL_SUPPORT
   SSL_CTX *ssl_ctx=NULL;
 #endif /* WITH_SSL_SUPPORT */
 
-  extern struct sockaddr_in *peername;
-  extern struct sockaddr_in *sockname;
-  extern struct utsname *system_uname;
-  extern options_t opt;
-
   /*
    * Create Logger
    */
@@ -405,9 +440,9 @@ int main (int argc, char **argv, char **envp){
 		  &(opt.mangled_capability), opt.ssl_mode, tls_state);
 
   if(opt.ssl_mode & SSL_LISTEN_MASK) {
-    ssl_ctx = perdition_ssl_ctx(NULL, NULL, opt.ssl_cert_file, 
-		    opt.ssl_key_file, opt.ssl_ca_chain_file,
-		    opt.ssl_listen_ciphers);
+    ssl_ctx = perdition_ssl_ctx(opt.ssl_ca_file, opt.ssl_ca_path,
+				opt.ssl_cert_file, opt.ssl_key_file,
+				opt.ssl_ca_chain_file, opt.ssl_listen_ciphers);
     if(!ssl_ctx) {
       PERDITION_DEBUG_SSL_ERR("perdition_ssl_ctx");
       VANESSA_LOGGER_ERR("Fatal error establishing SSL context for listening");
@@ -459,12 +494,14 @@ int main (int argc, char **argv, char **envp){
   /*
    * Allocate the peername and sockname structures
    */
-  if((sockname=(struct sockaddr_in *)malloc(sizeof(struct sockaddr_in)))==NULL){
+  sockname = malloc(sizeof(*sockname));
+  if (!sockname) {
     VANESSA_LOGGER_DEBUG_ERRNO("malloc sockname");
     VANESSA_LOGGER_ERR("Fatal error allocating memory. Exiting.");
     perdition_exit_cleanly(-1);
   }
-  if((peername=(struct sockaddr_in *)malloc(sizeof(struct sockaddr_in)))==NULL){
+  peername = malloc(sizeof(*peername));
+  if (!peername) {
     VANESSA_LOGGER_DEBUG_ERRNO("malloc peername");
     VANESSA_LOGGER_ERR("Fatal error allocating memory. Exiting.");
     perdition_exit_cleanly(-1);
@@ -473,13 +510,48 @@ int main (int argc, char **argv, char **envp){
 
   /* Open incoming socket as required */
   if(!opt.inetd_mode) {
-    g = vanessa_socket_server_bind(opt.listen_port, opt.bind_address, 
- 		    		   opt.no_lookup?VANESSA_SOCKET_NO_LOOKUP:0);
-    if(g < 0) {
-      VANESSA_LOGGER_DEBUG("vanessa_socket_server_bind");
-      VANESSA_LOGGER_ERR("Fatal error listening for connections. Exiting.");
-      perdition_exit_cleanly(-1);
-    }
+	  size_t nfrom;
+	  char **fromv;
+	  
+  	  if (opt.bind_address) 
+		  nfrom = vanessa_dynamic_array_get_count(opt.bind_address);
+	  else 
+		  nfrom = 1;
+	  
+  	  fromv = (char **)malloc(((nfrom * 2) + 1) * sizeof(char *));
+	  if (!fromv) {
+		  VANESSA_LOGGER_DEBUG_ERRNO("malloc fromv");
+		  VANESSA_LOGGER_ERR("Fatal error allocating memory. Exiting.");
+		  perdition_exit_cleanly(-1);
+	  }
+	  
+  	  if (opt.bind_address) {
+		  size_t i;
+		  user_server_port_t *from_usp;
+		  for (i = 0; i < nfrom; i++) {
+			  from_usp = vanessa_dynamic_array_get_element(
+						opt.bind_address, i);
+			  fromv[i * 2] = user_server_port_get_server(from_usp);
+			  fromv[(i * 2) + 1] = user_server_port_get_port(
+			  			from_usp);
+			  if (!fromv[(i * 2) + 1])
+				  fromv[(i * 2) + 1] = opt.listen_port;
+		  }
+	  }
+	  else {
+	  	  fromv[0] = "0.0.0.0";
+	  	  fromv[1] = opt.listen_port;
+	  }
+	  fromv[nfrom * 2] = NULL;
+	  g = vanessa_socket_server_bindv(fromv, 
+	  			opt.no_lookup?VANESSA_SOCKET_NO_LOOKUP:0);
+	  free(fromv);
+	  if(!g) {
+		  VANESSA_LOGGER_DEBUG("vanessa_socket_server_bindv");
+		  VANESSA_LOGGER_ERR("Fatal error listening for connections."
+				     "Exiting.");
+		  perdition_exit_cleanly(-1);
+	  }
   }
 
   /*
@@ -499,27 +571,31 @@ int main (int argc, char **argv, char **envp){
 
   /* Get an incoming connection */
   if(opt.inetd_mode){
-    int namelen;
+    socklen_t namelen;
 
     if((client_io=io_create_fd(0, 1, PERDITION_LOG_STR_CLIENT))==NULL){
       VANESSA_LOGGER_DEBUG("io_create_fd 1");
       VANESSA_LOGGER_ERR("Fatal error setting IO. Exiting.");
       perdition_exit_cleanly(-1);
     }
+    io_set_timeout(client_io, opt.authenticate_timeout);
 
     namelen = sizeof(*peername);
-    if(getpeername(0, (struct sockaddr *)peername, &namelen)){
+    if (getpeername(0, (struct sockaddr *)peername, &namelen)) {
+      free(peername);
       peername=NULL;
     }
 
     namelen = sizeof(*sockname);
-    if(getsockname(1, (struct sockaddr *)sockname, &namelen)){
+    if (getsockname(1, (struct sockaddr *)sockname, &namelen)) {
+      free(sockname);
       sockname=NULL;
     }
   }
   else{
-    s = vanessa_socket_server_accept(g, opt.connection_limit, peername, 
-          sockname, 0);
+    s = vanessa_socket_server_acceptv(g, opt.connection_limit,
+				      (struct sockaddr *) peername,
+				      (struct sockaddr *) sockname, 0);
     if(s < 0){
       VANESSA_LOGGER_DEBUG("vanessa_socket_server_accept");
       VANESSA_LOGGER_ERR("Fatal error accepting child connection. Exiting.");
@@ -537,6 +613,7 @@ int main (int argc, char **argv, char **envp){
       VANESSA_LOGGER_ERR("Fatal error setting IO. Exiting.");
       perdition_exit_cleanly(-1);
     }
+    io_set_timeout(client_io, opt.authenticate_timeout);
   }
 
   /* A child process, or process handling an inetd connection
@@ -545,25 +622,44 @@ int main (int argc, char **argv, char **envp){
   signal(SIGPIPE,   perdition_exit_cleanly);
 
   /* Get the source and destination ip address as a string */
-  if(peername!=NULL){
-    snprintf(from_str, 17, "%s", inet_ntoa(peername->sin_addr));
+  if (peername!=NULL) {
+    rc = getnameinfo((struct sockaddr *)peername,
+		     perdition_get_salen((struct sockaddr *)peername),
+		     from_host_str, NI_MAXHOST, from_serv_str, NI_MAXSERV,
+		     NI_NUMERICHOST|NI_NUMERICSERV);
+    if (rc) {
+        VANESSA_LOGGER_DEBUG_UNSAFE("getnameinfo peername: %s",
+				    gai_strerror(rc));
+	VANESSA_LOGGER_ERR("Fatal error formatting peername");
+	perdition_exit_cleanly(-1);
+    }
   }
   else {
-    *from_str='\0';
+    *from_host_str='\0';
+    *from_serv_str='\0';
   }
-  if(sockname!=NULL){
-    snprintf(to_str,   17, "%s", inet_ntoa(sockname->sin_addr));
-    to_addr=&(sockname->sin_addr);
+  if (sockname!=NULL) {
+    rc = getnameinfo((struct sockaddr *)sockname,
+		     perdition_get_salen((struct sockaddr *)sockname),
+		     to_host_str, NI_MAXHOST, to_serv_str, NI_MAXSERV,
+		     NI_NUMERICHOST|NI_NUMERICSERV);
+    if (rc) {
+        VANESSA_LOGGER_DEBUG_UNSAFE("getnameinfo sockname: %s",
+				    gai_strerror(rc));
+	VANESSA_LOGGER_ERR("Fatal error formatting sockname");
+	perdition_exit_cleanly(-1);
+    }
   }
   else {
-    *to_str='\0';
-    to_addr=NULL;
+    *to_host_str='\0';
+    *from_host_str='\0';
   }
+  *from_to_host_str='\0';
   if(peername!=NULL && sockname!=NULL){
-    snprintf(from_to_str, 36, "%s->%s ", from_str, to_str);
-  }
-  else{
-    *from_to_str='\0';
+    strcat(from_to_host_str, " ");
+    strcat(from_to_host_str, from_host_str);
+    strcat(from_to_host_str, "->");
+    strcat(from_to_host_str, to_host_str);
   }
 
   /*Seed rand*/
@@ -572,11 +668,11 @@ int main (int argc, char **argv, char **envp){
 
   /*Log the session and change the proctitle*/
   if(opt.inetd_mode) {
-    VANESSA_LOGGER_INFO_UNSAFE("Connect: %sinetd_pid=%d", 
-          from_to_str, getppid());
+    VANESSA_LOGGER_INFO_UNSAFE("Connect:%s inetd_pid=%d",
+          from_to_host_str, getppid());
   }
   else {
-    VANESSA_LOGGER_INFO_UNSAFE("Connect: %s", from_to_str);
+    VANESSA_LOGGER_INFO_UNSAFE("Connect:%s", from_to_host_str);
   }
   set_proc_title("%s: connect", progname);
 
@@ -594,10 +690,8 @@ int main (int argc, char **argv, char **envp){
   /*Speak to our client*/
   if(greeting(client_io, protocol, GREETING_ADD_NODENAME)){
     VANESSA_LOGGER_DEBUG("greeting");
-    VANESSA_LOGGER_ERR_UNSAFE(
-      "Fatal error writing to client. %sExiting child.",
-      from_to_str
-    );
+    VANESSA_LOGGER_ERR_UNSAFE("Fatal error writing to client%s."
+			      "Exiting child.", from_to_host_str);
     perdition_exit_cleanly(-1);
   }
 
@@ -610,19 +704,20 @@ int main (int argc, char **argv, char **envp){
     token_flush();
     if(status<0){
       VANESSA_LOGGER_DEBUG("protocol->in_get_pw");
-      VANESSA_LOGGER_ERR_UNSAFE(
-	"Fatal Error reading authentication information from client \"%s\": "
-	"Exiting child", 
-	from_to_str
-      );
+      if (io_get_err(client_io) == io_err_timeout)
+	VANESSA_LOGGER_ERR_UNSAFE("Fatal Error: Timeout reading "
+				  "authentication information from "
+				  "client%s: Exiting child",
+				  from_to_host_str);
+      else
+	VANESSA_LOGGER_ERR_UNSAFE("Fatal Error reading authentication "
+				  "information from client%s: "
+				  "Exiting child", from_to_host_str);
       perdition_exit_cleanly(-1);
     }
     else if(status == 1){
-      VANESSA_LOGGER_ERR_UNSAFE(
-        "Closing NULL session: %susername=%s", 
-        from_to_str,
-        str_null_safe(pw.pw_name)
-      );
+      VANESSA_LOGGER_ERR_UNSAFE("Closing NULL session:%s username=%s",
+				from_to_host_str, str_null_safe(pw.pw_name));
       perdition_exit_cleanly(0);
     }
 #ifdef WITH_SSL_SUPPORT
@@ -643,27 +738,26 @@ int main (int argc, char **argv, char **envp){
 		    ((status == 0) && (opt.ssl_mode & SSL_MODE_TLS_LISTEN) &&
 		    (opt.ssl_mode & SSL_MODE_TLS_LISTEN_FORCE) &&
 		    !(tls_state & SSL_MODE_TLS_LISTEN))) {
-	    LOGIN_FAILED(PROTOCOL_NO, "Login Disabled");
+	    LOGIN_FAILED_PROTOCOL(PROTOCOL_NO, "Login Disabled");
 	    PERDITION_CLEAN_UP_MAIN;
 	    continue;
     }
 #endif /* WITH_SSL_SUPPORT */
 
-    if((username=username_mangle(pw.pw_name, to_addr, STATE_GET_SERVER))==NULL){
+    username = username_mangle(pw.pw_name, STATE_GET_SERVER);
+    if (!username) {
       VANESSA_LOGGER_DEBUG("username_mangle STATE_GET_SERVER");
-      VANESSA_LOGGER_ERR_UNSAFE(
-	"Fatal error manipulating username for client \"%s\": Exiting child",
-	from_str
-      );
+      VANESSA_LOGGER_ERR_UNSAFE("Fatal error manipulating username "
+				"for client \"%s\": Exiting child",
+				from_host_str);
       perdition_exit_cleanly(-1);
     }
 
     /*Read the server from the map, if we have a map*/
     if(dbserver_get || dbserver_get2 || opt.client_server_specification) {
-    	usp = getserver(username, from_str, to_str, 
-			       peername==NULL?0:ntohs(peername->sin_port), 
-			       sockname==NULL?0:ntohs(sockname->sin_port), 
-			       dbserver_get, dbserver_get2);
+	usp = getserver(username, from_host_str, to_host_str,
+			from_serv_str, to_serv_str,
+			dbserver_get, dbserver_get2);
     }
     if(usp){
       port = usp->port;
@@ -676,11 +770,9 @@ int main (int argc, char **argv, char **envp){
         }
         if((pw.pw_name=strdup(usp->user))==NULL){
 	    VANESSA_LOGGER_DEBUG_ERRNO("strdup");
-            VANESSA_LOGGER_ERR_UNSAFE(
-	      "Fatal error manipulating username for client \"%s\": "
-	      "Exiting child",
-	      from_str
-            );
+            VANESSA_LOGGER_ERR_UNSAFE("Fatal error manipulating username "
+				      "for client \"%s\": Exiting child",
+				      from_host_str);
         }
       }
     }
@@ -702,20 +794,19 @@ int main (int argc, char **argv, char **envp){
 
     /*Try again if we didn't get anything useful*/
     if(!servername || !*servername) {
-	    LOGIN_FAILED(PROTOCOL_ERR, "Could not determine server");
+	    LOGIN_FAILED_PROTOCOL(PROTOCOL_ERR, "Could not determine server");
 	    PERDITION_CLEAN_UP_MAIN;
 	    continue;
     }
 
 #ifdef WITH_PAM_SUPPORT
     if(opt.authenticate_in){
-      if((pw2.pw_name=username_mangle(pw.pw_name, 
-            to_addr, STATE_LOCAL_AUTH))==NULL){
+      pw2.pw_name = username_mangle(pw.pw_name, STATE_LOCAL_AUTH);
+      if (!pw2.pw_name) {
         VANESSA_LOGGER_DEBUG("username_mangle STATE_LOCAL_AUTH");
-        VANESSA_LOGGER_ERR_UNSAFE(
-	  "Fatal error manipulating username for client \"%s\": Exiting child",
-	  from_str
-        );
+        VANESSA_LOGGER_ERR_UNSAFE("Fatal error manipulating username for "
+				  "client \"%s\": Exiting child",
+				  from_host_str);
         perdition_exit_cleanly(-1);
       }
       pw2.pw_passwd=pw.pw_passwd;
@@ -725,9 +816,7 @@ int main (int argc, char **argv, char **envp){
         VANESSA_LOGGER_INFO(
 	  "Local authentication failure for client: Allowing retry."
         );
-  	VANESSA_LOGGER_LOG_AUTH(auth_log, from_to_str, pw.pw_name, 
-			servername, port, 
-			"failed: local authentication failure");
+  	PERDITION_LOG_AUTH("failed: local authentication failure");
         PERDITION_CLEAN_UP_MAIN;
         continue;
       }
@@ -748,12 +837,12 @@ int main (int argc, char **argv, char **envp){
 #endif /* WITH_PAM_SUPPORT */
 
     /* Talk to the real pop server for the client*/
-    s = vanessa_socket_client_src_open(opt.bind_address, NULL, servername, 
-				    port, 
-				    (opt.no_lookup?VANESSA_SOCKET_NO_LOOKUP:0));
+    s = vanessa_socket_client_src_open(NULL, NULL, servername, port,
+				       opt.no_lookup ? 
+				       VANESSA_SOCKET_NO_LOOKUP : 0);
     if(s < 0) {
 	    VANESSA_LOGGER_DEBUG("vanessa_socket_client_open");
-	    LOGIN_FAILED(PROTOCOL_ERR, "Could not connect to server");
+	    LOGIN_FAILED_PROTOCOL(PROTOCOL_ERR, "Could not connect to server");
 	    PERDITION_CLEAN_UP_MAIN;
 	    continue;
     }
@@ -763,6 +852,7 @@ int main (int argc, char **argv, char **envp){
       VANESSA_LOGGER_ERR("Fatal error setting IO. Exiting.");
       perdition_exit_cleanly(-1);
     }
+    io_set_timeout(server_io, opt.authenticate_timeout);
 
 #ifdef WITH_SSL_SUPPORT
     if(opt.ssl_mode & SSL_MODE_SSL_OUTGOING) {
@@ -777,13 +867,12 @@ int main (int argc, char **argv, char **envp){
 #endif /* WITH_SSL_SUPPORT */
 
     /* Authenticate the user with the pop server */
-    if((pw2.pw_name=username_mangle(pw.pw_name, 
-          to_addr, STATE_REMOTE_LOGIN))==NULL){
+    pw2.pw_name = username_mangle(pw.pw_name, STATE_REMOTE_LOGIN);
+    if(!pw2.pw_name) {
       VANESSA_LOGGER_DEBUG("username_mangle STATE_REMOTE_LOGIN");
-      VANESSA_LOGGER_ERR_UNSAFE(
-	"Fatal error manipulating username for client \"%s\": Exiting child",
-	from_str
-      );
+      VANESSA_LOGGER_ERR_UNSAFE("Fatal error manipulating username "
+				"for client \"%s\": Exiting child",
+				from_host_str);
       perdition_exit_cleanly(-1);
     }
     pw2.pw_passwd=pw.pw_passwd;
@@ -792,7 +881,7 @@ int main (int argc, char **argv, char **envp){
 		    protocol);
     if(status==0){
 	    quit(server_io, protocol, our_tag);
-	    LOGIN_FAILED(PROTOCOL_NO, "Connection Negotiation Failure");
+	    LOGIN_FAILED_PROTOCOL(PROTOCOL_NO, "Connection Negotiation Failure");
 	    PERDITION_CLEAN_UP_MAIN;
 	    continue;
     }
@@ -817,7 +906,7 @@ int main (int argc, char **argv, char **envp){
 		    (opt.ssl_mode & SSL_MODE_TLS_OUTGOING_FORCE) &&
 		    !(status & PROTOCOL_S_STARTTLS)) {
 	    quit(server_io, protocol, our_tag);
-	    LOGIN_FAILED(PROTOCOL_NO, "TLS not present");
+	    LOGIN_FAILED_PROTOCOL(PROTOCOL_NO, "TLS not present");
 	    PERDITION_CLEAN_UP_MAIN;
 	    continue;
     }
@@ -840,24 +929,26 @@ int main (int argc, char **argv, char **envp){
                 VANESSA_LOGGER_ERR("Fatal error writing to client. Exiting child.");
                 perdition_exit_cleanly(-1);
               }
-              VANESSA_LOGGER_LOG_AUTH(auth_log, from_to_str, pw.pw_name,
-                                      servername, port,
-                                      "failed: Re-Authentication Failure");
+              PERDITION_LOG_AUTH("failed: Re-Authentication Failure");
             }
             else
-              LOGIN_FAILED(PROTOCOL_NO, "Re-Authentication Failure");
+              LOGIN_FAILED_PROTOCOL(PROTOCOL_NO, "Re-Authentication Failure");
 	    PERDITION_CLEAN_UP_MAIN;
 	    continue;
     }
     if(status==2){
 	    sleep(VANESSA_LOGGER_ERR_SLEEP);
-	    LOGIN_FAILED(PROTOCOL_NO, "Login Disabled");
+	    LOGIN_FAILED_PROTOCOL(PROTOCOL_NO, "Login Disabled");
 	    PERDITION_CLEAN_UP_MAIN;
 	    continue;
     }
     else if(status<0){
       VANESSA_LOGGER_DEBUG_UNSAFE("protocol->out_authenticate %d", status);
-      VANESSA_LOGGER_ERR("Fatal error authenticating user. Exiting child.");
+      if (io_get_err(client_io) == io_err_timeout)
+	VANESSA_LOGGER_ERR("Fatal error: Timeout authenticating user. "
+			   "Exiting child.");
+      else
+        VANESSA_LOGGER_ERR("Fatal error authenticating user. Exiting child.");
       perdition_exit_cleanly(-1);
     }
 
@@ -882,8 +973,7 @@ int main (int argc, char **argv, char **envp){
     break;
   }
 
-  VANESSA_LOGGER_LOG_AUTH(auth_log, from_to_str, pw.pw_name, 
-		  servername, port, "ok");
+  PERDITION_LOG_AUTH("ok");
 
   if(opt.server_resp_line){
      free(server_resp_buf);
@@ -897,21 +987,24 @@ int main (int argc, char **argv, char **envp){
   }
 
   /*Let the client talk to the real server*/
-  if(io_pipe(server_io, client_io, buffer, BUFFER_SIZE, opt.timeout,
+  io_set_timeout(client_io, opt.timeout);
+  io_set_timeout(server_io, opt.timeout);
+  if(io_pipe(server_io, client_io, buffer, BUFFER_SIZE,
         &bytes_written, &bytes_read, &auth_log)<0){
     VANESSA_LOGGER_DEBUG("vanessa_socket_pipe");
-    VANESSA_LOGGER_ERR("Fatal error piping data. Exiting child.");
+    if (io_get_err(client_io) == io_err_timeout)
+      VANESSA_LOGGER_ERR("Fatal error: Timeout piping data. "
+			 "Exiting child.");
+    else
+      VANESSA_LOGGER_ERR("Fatal error piping data. Exiting child.");
     perdition_exit_cleanly(-1);
   }
 
   /*Time to leave*/
-  VANESSA_LOGGER_INFO_UNSAFE(
-    "Close: %suser=\"%s\" received=%d sent=%d", 
-    str_null_safe(from_to_str),
-    str_null_safe(pw.pw_name),
-    bytes_read,
-    bytes_written
-  );
+  VANESSA_LOGGER_INFO_UNSAFE("Close:%s user=\"%s\" received=%d sent=%d",
+			     str_null_safe(from_to_host_str),
+			     str_null_safe(pw.pw_name),
+			     bytes_read, bytes_written);
   set_proc_title("%s: close", progname);
 
   getserver_closelib(handle);
@@ -935,8 +1028,6 @@ int main (int argc, char **argv, char **envp){
  **********************************************************************/
 
 static void perdition_reread_handler(int sig){
-  extern options_t opt;
-
   if(vanessa_logger_reopen(vanessa_logger_get()) < 0) {
     fprintf(stderr, "Fatal error reopening logger. Exiting.");
     perdition_exit_cleanly(-1);
@@ -977,8 +1068,6 @@ static void perdition_reread_handler(int sig){
 static void 
 perdition_exit_cleanly(int i) 
 {
-	extern options_t opt;
-
      	if (pid_file && (unlink(opt.pid_file) < 0)) {
 		VANESSA_LOGGER_INFO_UNSAFE("Could not remove pid file "
 				"[%s]: %s\n", opt.pid_file,
@@ -1001,7 +1090,24 @@ perdition_chown(const char *path, const char *username, const char *group)
 		gid = (gid_t) atoi(group);
 	} else {
 		if ((gr = getgrnam(group)) == NULL) {
-			VANESSA_LOGGER_DEBUG_ERRNO("getgrnam");
+			switch (errno) {
+			case 0:
+			case ENOENT:
+			case ESRCH:
+			case EBADF:
+			case EPERM:
+				VANESSA_LOGGER_DEBUG_UNSAFE("getgrnam: "
+							    "\"%s\": "
+							    "not found",
+							    group);
+				break;
+			default:
+				VANESSA_LOGGER_DEBUG_UNSAFE("getgrnam: "
+							    "\"%s\": %s",
+							    group,
+							    strerror(errno));
+				break;
+			}
 			return (-1);
 		}
 		gid = gr->gr_gid;
