@@ -17,11 +17,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -59,8 +58,6 @@ static char *pop3s_capability(char *capability, char **mangled_capability,
 char *pop3s_type[]={POP3_OK, POP3_ERR, POP3_ERR};
 
 protocol_t *pop3s_initialise_protocol(protocol_t *protocol){
-  extern char *pop3s_type[];
-
   protocol->type = pop3s_type;
   protocol->write = pop3_write;
   protocol->greeting_string = POP3_GREETING;
