@@ -21,11 +21,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -42,6 +41,7 @@
 #include <termios.h>
 #include <string.h>
 #include <openssl/bio.h>
+#include <openssl/x509v3.h>
 
 #include "ssl.h"
 #include "log.h"
@@ -52,6 +52,8 @@
 #include <dmalloc.h>
 #endif
 
+#include "perdition_globals.h"
+
 #define PERDITION_SSL_CLIENT (flag_t) 0x1
 #define PERDITION_SSL_SERVER (flag_t) 0x2
 
@@ -157,23 +159,57 @@ __perdition_ssl_passwd_cb(char *buf, int size, int rwflag, void *data)
  *
  **********************************************************************/
 
+static int
+__perdition_verify_callback_debug(X509 *cert, int depth)
+{
+	char *tmp_str;
+	BIO *tmp_bio;
+	long len;
+	int status = 0;
+
+	if (!opt.debug)
+		return 1;
+
+	tmp_bio = BIO_new(BIO_s_mem());
+	if (!tmp_bio) {
+		VANESSA_LOGGER_DEBUG("BIO_new");
+		return 0;
+	}
+
+	X509_NAME_print_ex(tmp_bio, X509_get_subject_name(cert),
+			   3, XN_FLAG_ONELINE);
+	len = BIO_get_mem_data(tmp_bio, &tmp_str);
+
+	tmp_str = strn_to_str(tmp_str, len);
+	if (!tmp_str) {
+		VANESSA_LOGGER_DEBUG("BIO_new");
+		goto err;
+	}
+
+	VANESSA_LOGGER_DEBUG_RAW_UNSAFE("depth:%d cert:\"%s\"", depth, tmp_str);
+
+	str_free(tmp_str);
+	status = 1;
+err:
+	if (!BIO_free(tmp_bio)) {
+		VANESSA_LOGGER_DEBUG("BIO_free");
+		return 0;
+	}
+	return status;
+}
+
 static int 
 __perdition_verify_callback(int ok, X509_STORE_CTX *ctx)
 {
-	X509 *cert;
+	X509 *cert = X509_STORE_CTX_get_current_cert(ctx);
+	int depth = X509_STORE_CTX_get_error_depth(ctx);
 
-	extern options_t opt;
-
-	cert = X509_STORE_CTX_get_current_cert(ctx);
-	if (opt.debug) {
-		char buf[MAX_LINE_LENGTH];
-		X509_NAME_oneline(X509_get_subject_name(cert),
-				buf, MAX_LINE_LENGTH);
-        	VANESSA_LOGGER_DEBUG_RAW_UNSAFE("depth:%d cert:\"%s\"", 
-				X509_STORE_CTX_get_error_depth(ctx), buf);
+	if (!__perdition_verify_callback_debug(cert, depth)) {
+		VANESSA_LOGGER_DEBUG("__perdition_verify_callback_debug");
+		return 0;
 	}
 
-	if (opt.ssl_cert_verify_depth < X509_STORE_CTX_get_error_depth(ctx)) {
+	if (opt.ssl_cert_verify_depth < depth) {
 		VANESSA_LOGGER_DEBUG_UNSAFE("Chain too long, try adjusting "
 				"ssl_cert_verify_depth: %d > %d",
 				X509_STORE_CTX_get_error_depth(ctx), 
@@ -195,24 +231,53 @@ __perdition_verify_callback(int ok, X509_STORE_CTX *ctx)
 	X509_NAME_oneline((_value), buf, MAX_LINE_LENGTH);                  \
 	VANESSA_LOGGER_DEBUG_RAW_UNSAFE("%s:\"%s\"", (_key), buf);
 
-#define __PERDITION_VERIFY_RESULT_TIME(_key, _time, _err)                   \
-{                                                                           \
-	BIO *tmp_bio;                                                       \
-	char *tmp_str;                                                      \
-	tmp_bio = BIO_new(BIO_s_mem());                                     \
-	if (!tmp_bio) {                                                     \
-		VANESSA_LOGGER_DEBUG("BIO_new");                            \
-		verify = (_err);                                            \
-	}                                                                   \
-	ASN1_TIME_print(tmp_bio, (_time));                                  \
-	BIO_get_mem_data(tmp_bio, &tmp_str);                                \
-	VANESSA_LOGGER_DEBUG_RAW_UNSAFE("%s:\"%s\"", (_key), tmp_str);      \
-	if (!BIO_free(tmp_bio)) {                                           \
-		VANESSA_LOGGER_DEBUG("BIO_free");                           \
-		verify = (_err);                                            \
-	}                                                                   \
+static char *
+__perdition_verify_result_time(const char *key, ASN1_TIME *time)
+{
+	BIO *tmp_bio;
+	char *tmp_str;
+	long len;
+	int status = 0;
+
+	tmp_bio = BIO_new(BIO_s_mem());
+	if (!tmp_bio) {
+		VANESSA_LOGGER_DEBUG("BIO_new");
+		return NULL;
+	}
+
+	ASN1_TIME_print(tmp_bio, time);
+	len = BIO_get_mem_data(tmp_bio, &tmp_str);
+
+	tmp_str = strn_to_str(tmp_str, len);
+	if (!tmp_str) {
+		VANESSA_LOGGER_DEBUG("BIO_free");
+		goto err;
+	}
+
+
+	VANESSA_LOGGER_DEBUG_RAW_UNSAFE("%s:\"%s\"", key, tmp_str);
+
+	status = 1;
+err:
+	if (!BIO_free(tmp_bio)) {
+		VANESSA_LOGGER_DEBUG("BIO_free");
+		str_free(tmp_str);
+		return NULL;
+	}
+	return tmp_str;
 }
 
+#define __PERDITION_VERIFY_RESULT_TIME(_key, _time, _err)		    \
+do {									    \
+	char *tmp_str = __perdition_verify_result_time(_key, _time);	    \
+	if (!tmp_str)							    \
+		verify = _err;						    \
+	else {								    \
+		VANESSA_LOGGER_DEBUG_RAW_UNSAFE("%s:\"%s\"", _key, tmp_str);\
+		str_free(tmp_str);					    \
+	}								    \
+} while (0)
+
 #define __PERDITION_VERIFY_RESULT_WARN(_msg)                                \
 	 VANESSA_LOGGER_DEBUG_RAW("warning: " _msg)
 	
@@ -223,8 +288,6 @@ static long __perdition_verify_result(long verify, X509 *cert)
 {
 	char buf[MAX_LINE_LENGTH];
 
-	extern options_t opt;
-
 	/*
 	 * Handle all error codes. See verify(1) 
 	 */
@@ -412,8 +475,6 @@ SSL_CTX *perdition_ssl_ctx(const char *ca_file, const char *ca_path,
 	const char *use_ca_file = NULL;
 	const char *use_ca_path = NULL;
 
-	extern options_t opt;
-
 	/* 
 	 * If either the certificate or private key is non-NULL the
 	 * other should be too
@@ -524,11 +585,160 @@ SSL_CTX *perdition_ssl_ctx(const char *ca_file, const char *ca_path,
 }
 
 
+/**********************************************************************
+ * __perdition_ssl_compare
+ * pre: key:  name to match, null terminated
+ *      val: value to match against
+ *      val_len: length of value in bytes
+ * post: none
+ * return: 0 on success
+ *         -2 if the common key does not match
+ **********************************************************************/
+
+static int
+__perdition_ssl_compare_key(const char *key, const void *val, size_t val_len)
+{
+	const char *domain;
+
+	if(strlen(key) == val_len && !memcmp(key, val, val_len))
+		return 0;
+
+	/* A wild card common name is allowed
+	 * It should be of the form *.domain */
+	if (!memcmp(val, "*.", 2)) {
+	        domain = strchr(key, '.');
+		if (domain && strlen(domain) == val_len - 1 &&
+		    !memcmp(domain, val + 1, val_len - 1))
+			return 0;
+	}
+
+	return -2;
+}
+
+
 /**********************************************************************
  * __perdition_ssl_check_common_name
- * Log the details of a certificate
- * pre: cert: server certificate to check the common name of
- *      server: server name to match against the common name
+ * pre: cert: certificate to check the common names of
+ *      key:  name to match
+ * post: none
+ * return: 0 on success
+ *         -1 on error
+ *         -2 if the common name does not match
+ **********************************************************************/
+
+static int 
+__perdition_ssl_check_common_name(X509 *cert, const char *key)
+{
+	int i;
+	size_t key_len;
+	X509_NAME_ENTRY *e;
+	X509_NAME *name;
+
+	name = X509_get_subject_name(cert);
+	if (!name) {
+		VANESSA_LOGGER_DEBUG_RAW("warning: could not extract "
+					 "name from certificate");
+		return -1;
+	}
+
+	key_len = strlen(key);
+
+	i = -1;
+	while (1) {
+		i = X509_NAME_get_index_by_NID(name, NID_commonName, i);
+		if (i == -1)
+			break;
+
+		e = X509_NAME_get_entry(name, i);
+		if (!e) {
+			VANESSA_LOGGER_DEBUG_RAW_UNSAFE("warning: could not "
+				"extract name entry %d from certificate", i);
+			return -1;
+		}
+
+		if (!__perdition_ssl_compare_key(key, e->value->data,
+						 e->value->length))
+			return 0;
+	}
+
+	return -2;
+}
+
+
+/**********************************************************************
+ * __perdition_ssl_check_alt_subject
+ * pre: cert: certificate to check the names of
+ *      key:  name to match
+ * post: none
+ * return: 0 on success
+ *         -1 on error
+ *         -2 if the common name does not match
+ **********************************************************************/
+
+static int
+__perdition_ssl_check_alt_subject(X509 *cert, const char *key)
+{
+	int i, count, len, status = 0;
+	char *str;
+	BIO *bio;
+	GENERAL_NAME *gn;
+	STACK_OF(GENERAL_NAME) *gns;
+
+	gns = (GENERAL_NAMES*)X509_get_ext_d2i(cert,
+			NID_subject_alt_name, NULL, NULL);
+	if (!gns) {
+		VANESSA_LOGGER_DEBUG_RAW("warning: could not "
+			"extract extensions from certificate");
+		return -1;
+	}
+
+	bio = BIO_new(BIO_s_mem());
+	if (!bio) {
+		VANESSA_LOGGER_DEBUG("BIO_new");
+		return -1;
+	}
+
+	count = sk_GENERAL_NAME_num(gns);
+	for (i = 0; i < count; i++) {
+		gn = sk_GENERAL_NAME_value(gns, i);
+		if (!gn) {
+			VANESSA_LOGGER_DEBUG_RAW_UNSAFE("warning: "
+				"could not extract alt subject %d "
+				"from certificate", i);
+			status = -1;
+			goto out;
+		}
+
+		if (gn->type != GEN_DNS)
+			continue;
+
+		ASN1_STRING_print_ex(bio, gn->d.dNSName, ASN1_STRFLGS_RFC2253);
+		len = BIO_get_mem_data(bio, &str);
+
+		if (!__perdition_ssl_compare_key(key, str, len))
+			goto out;
+
+		if (BIO_reset(bio) != 1) {
+			VANESSA_LOGGER_DEBUG("BIO_reset");
+			status = -1;
+			goto out;
+		}
+	}
+
+	status = -2;
+out:
+	if (!BIO_free(bio)) {
+		VANESSA_LOGGER_DEBUG("BIO_free");
+		return -1;
+	}
+	return status;
+}
+
+
+/**********************************************************************
+ * __perdition_ssl_check_name
+ * pre: cert: certificate to check the common names and alt subjects of
+ *      key:  name to match
  * post: none
  * return: 0 on success
  *         -1 on error
@@ -536,39 +746,33 @@ SSL_CTX *perdition_ssl_ctx(const char *ca_file, const char *ca_path,
  *            not exist
  **********************************************************************/
 
-static int 
-__perdition_ssl_check_common_name(X509 *cert, const char *server)
+static int
+__perdition_ssl_check_name(X509 *cert, const char *key)
 {
-	char *domain;
-	char common_name[MAX_LINE_LENGTH];
+	int rc;
 
-	extern options_t opt;
-
-	if (opt.ssl_no_cn_verify || !server)
+	if (opt.ssl_no_cn_verify || !key)
 		return 0;
 
 	if (!cert) {
 		VANESSA_LOGGER_DEBUG_RAW("warning: no server certificate");
 		return -2;
 	}
-	
-	if (X509_NAME_get_text_by_NID(X509_get_subject_name(cert), 
-			NID_commonName, common_name, MAX_LINE_LENGTH) < 0) {
-		PERDITION_DEBUG_SSL_ERR("X509_NAME_get_text_by_OBJ");
+
+	rc = __perdition_ssl_check_common_name(cert, key);
+	if (rc == 0)
+	    return 0;
+	else if (rc == -1) {
+		VANESSA_LOGGER_DEBUG("__perdition_ssl_check_common_name");
 		return -1;
 	}
-	common_name[MAX_LINE_LENGTH -1] = '\0';
 
-	if(!strcmp(server, common_name))
-		return 0;
-
-	/* A wild card common name is allowed 
-	 * It should be of the form *.domain */
-	if (*common_name == '*' && *(common_name+1) == '.') {
-		domain = strchr(server, '.');
-		if (domain && !strcmp(common_name+2, domain+1)) {
-			return 0;
-		}
+	rc = __perdition_ssl_check_alt_subject(cert, key);
+	if (rc == 0)
+	    return 0;
+	else if (rc == -1) {
+		VANESSA_LOGGER_DEBUG("__perdition_ssl_check_alt_subject");
+		return -1;
 	}
 
 	VANESSA_LOGGER_DEBUG_RAW("error: common name mismatch");
@@ -592,8 +796,6 @@ __perdition_ssl_log_certificate(SSL *ssl, X509 *cert)
 {
 	char *str = NULL;
 
-	extern options_t opt;
-
 	if (!opt.debug)
 		return 0;
 
@@ -649,8 +851,6 @@ __perdition_ssl_check_certificate(io_t * io, const char *ca_file,
 	SSL *ssl;
 	int status = 0;
 
-	extern options_t opt;
-
 	ssl = io_get_ssl(io);
 	if (!ssl) {
 		VANESSA_LOGGER_DEBUG("vanessa_socket_get_ssl");
@@ -675,9 +875,9 @@ __perdition_ssl_check_certificate(io_t * io, const char *ca_file,
 		goto leave;
 	}
 
-	status = __perdition_ssl_check_common_name(cert, server);
+	status = __perdition_ssl_check_name(cert, server);
 	if (status < 0) {
-		VANESSA_LOGGER_DEBUG("__perdition_ssl_check_common_name");
+		VANESSA_LOGGER_DEBUG("__perdition_ssl_check_name");
 		goto leave;
 	}
 
@@ -726,6 +926,7 @@ static io_t *__perdition_ssl_connection(io_t *io, SSL_CTX *ssl_ctx,
 		goto bail;
 	}
 
+	io_set_timeout(new_io, io_get_timeout(io));
 	io_destroy(io);
 	io = NULL;
 
