@@ -17,11 +17,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  **********************************************************************/
 
@@ -49,6 +48,8 @@
 #include <dmalloc.h>
 #endif
 
+#include "perdition_globals.h"
+
 
 /**********************************************************************
  * strrstr
@@ -220,8 +221,6 @@ int str_vwrite(io_t * io, const flag_t flag, const size_t nargs,
 	const char *str;
 	int bytes = 0;
 
-	extern options_t opt;
-
 	str = __str_vwrite(io, flag, nargs, fmt, ap, &bytes);
 	if(!str) {
 		VANESSA_LOGGER_DEBUG("__str_vwrite");
@@ -231,7 +230,7 @@ int str_vwrite(io_t * io, const flag_t flag, const size_t nargs,
 	if (opt.connection_logging) {
 		char *dump_str;
 
-		dump_str = VANESSA_LOGGER_DUMP(str, bytes, 0);
+		dump_str = VANESSA_LOGGER_DUMP((unsigned char *)str, bytes, 0);
 		if (!dump_str) {
 			VANESSA_LOGGER_DEBUG("VANESSA_LOGGER_DUMP");
 			return (-1);
